; Module 'Coffre'
; Gestion des coffres pour eviter qu'il reste tout le temps sur la carte.

; Fonctions :
;	chest::init
;	chest::quit
;	chest::add
;	chest::is
;	chest::remove

;	chest::calculate
;	chest::save
;	chest::load

; Initialise la gestion des coffres.
; A appeler avant toute utilisation d'une quelconque fonction de ce module
; (Appeler automatiquement lors de la creation d'un jeu, donc en cours du jeu
; c'est inutile)
; In / Out : Nothing
; Destroy: d0-d2/a0-a1
chest::init
	move.w	#(CHESTS_MAX/8),d0
	bsr	Alloc
	move.w	d0,Chest_hd
	move.l	a0,Chests_adr(a6)
	moveq	#CHESTS_MAX/(8*4)-1,d0
\clr_loop:	clr.l	(a0)+
		dbf	d0,\clr_loop
	rts

Chest_hd	dc.w	0

; Libere tout du module 'chests'
; A appeler lors de la fin du jeu.
; In / Out / Destroy : nothing
chest::quit
	move.w	Chest_hd(Pc),d0
	bra	Free

; Enleve les coffres deja ouvert de la carte courante.
; Les coffres ouverts voient leur tile correspondant incremente de 1
; In :
;	Rien (va lire directement dans map.h)
; Out:
;	Rien
; Destroy:
;	Rien
chest::remove
	movem.l	d0/d1/d5/a1-a5,-(a7)

	move.l	Graph_adr(a6),a5
	move.l	Height_tab(a6),a4
	move.l	Items_list(a6),a3
	move.l	Chests_adr(a6),a2

	move.w	Map_size(a6),d5
	subq.w	#1,d5
\check_loop
		; Test s'il y a un item
		moveq	#0,d2
		move.b	(a4)+,d2
		cmp.b	#15,d2
		bcs.s	\nothing
			; Test si cet item est un coffre
			lsr.w	#3,d2	; Get no of items
			subq.w	#1,d2	; Skip the 5 firsts items
			lsl.w	#3,d2	; Get index of item
			lea	0(a3,d2.w),a1
			cmp.b	#ITEM_CHEST,item_sprite(a1) ; Item_type
			bne.s	\nothing
				; Test si le coffre a deja ete ouvert
				move.w	item_level(a1),d0
				move.w	d0,d1
				lsr.w	#3,d1
				and.w	#7,d0
				btst.b	d0,0(a2,d1.w)
				beq.s	\nothing
					addq.b	#1,(a5)	; Inc le numero du tile du coffre
					and.b	#%0111,-1(a4) ; Efface le test du coffre
\nothing:
		addq.l	#1,a5		; graph suivant
		dbf	d5,\check_loop
	movem.l	(a7)+,d0/d1/d5/a1-a5
	rts


; Ajoute le coffre dans la table de telle sorte que l'on 
; sache qu'il a ete ouvert
; In:
;	d0.w = No du coffre
;	Attention le numero est compris entre [0...CHESTS_MAX]
; Out:	d1.b = $00 => Ok
;	d1.b = $FF => Numero en dehors des limites
; Destroy:
;	d1.b
chest::add
	movem.l	d0/a0,-(a7)
	st.b	d1
	cmp.w	#CHESTS_MAX-1,d0
	bhi.s	\error
		move.w	d0,d1
		lsr.w	#3,d1
		and.w	#7,d0
		move.l	Chests_adr(a6),a0
		bset.b	d0,0(a0,d1.w)
		clr.b	d1
\error:	movem.l	(a7)+,d0/a0
	rts

; Teste si le coffre  a ete ouvert
; In:
;	d0.w = No du coffre
;	Attention le numero est compris entre [0...CHESTS_MAX]
; Out:	d1.b = $00 => Ouvert
;	d1.b = $FF => Non ouvert
; Destroy:
;	d1.b
chest::is
	movem.l	d0/a0,-(a7)
	clr.b	d1
	cmp.w	#CHESTS_MAX-1,d0
	bhi.s	\error
		move.w	d0,d1
		lsr.w	#3,d1
		and.w	#7,d0
		move.l	Chests_adr(a6),a0
		btst.b	d0,0(a0,d1.w)
		bne.s	\error
			st.b	d1
\error:	movem.l	(a7)+,d0/a0
	rts
