; /*************************************************************\
;|			Chrono Fantasy				|
;|		Original game (c) 1995 Square co		|
;| Adaptated to Ti89, Ti92II/+ by				|
;|			 Patrick Pelissier & Brian Tribondeau	|
; \*************************************************************/


; Constantes globales
VERSION		EQU	'2.80'			; No de version courante

DEBUG		EQU	0

	xdef	_main
	xdef	_comment

	; Include main header files
	include	tios.h
	include os.h		; Add else 92+/89 or 92 data
	
	; Include libraries header files
	include genlib.h
	include filelib.h
	include shrnklib.h	
	
	; Include all header files
	include "Chrono.h"		; Gestion des structures / de la memoire

	include "player.h"		; Description des joueurs
	include "print.h"		; Donnes sur l'affichage
	include "game.h"		; Donnes sur la gestion generale du jeu
	include "map.h"			; Donnes sur la carte
	include "dialog.h"		; Donnes sur les dialogues
	include "sprite.h"		; Donnes sur les sprites
	include "search.h"		; Donnes sur les fonctions de recherche
	include "object.h"		; Donnes sur les objets
	include "chests.h"		; Donnes sur les coffres
	include "routines.h"		; Donnes internes des routines
	include "rom.h"			; Donnes sur la description de la ROM.
	include "member.h"		; Donnes sur la gestion des membres
	include "tiles.h"		; Donnes sur le gestionnaire de tiles
	include "script.h"		; Donnes sur l'interpreteur de script
	include "event.h"		; Donnes sur les evenements
	include "fight.h"		; Donnes sur les combats
;	include "metadata.h"		; Donnes sur les meta-donnes
;	include "attack.h"		; Donnes sur les attaques
	include "items.h"
	include "console.h"		; Donnes su le mode console de commande
	include "macros.h"

_main:
	jsr	genlib::init		; Initialisation of GenLib
	bsr	crash::install		; Install Anti-Crash protection
	ALLOCATE_MEM			; Allocate memory on the stack and set a6

	; Fargo or Doors ?
	DOORS <move.b	CALCULATOR,Calc(a6)>	; Not Fargo
	FARGO <move.b	#2,Calc(a6)>		; Fargo
	move.w	#_FIRST_CALL,First_call

	; Init the DScreens
	genlib::PUSH_DSCREEN	d0	; Allocate a DSCREEN on the stack (7688 bytes)
	move.l	d0,Screen_1(a6)		; Save the adress of the first screen 
	move.l	d0,Current_dscreen(a6)	; Select this screen as the current
	jsr	genlib::set_dscreen_int		; Set the interrupt to this Dscreen
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen
	jsr	genlib::cls			; Clear it
	jsr	genlib::init_dscreen	; Allocate a new DScreen for double buffer
	move.l	a0,Screen_2(a6)		; Save the adresss of the 2nd screen
	clr.w	Select_s(a6)		; Select the 1st screen
	move.w	d0,DScreen_hd		; Save the handle and test if it =0
	beq.s	_quit			; Memory ?

	; Resolve external files
	bsr	search::rom

	; Start a new game
	bsr	game::new
	bsr	game::play
_quit	bsr	game::destroy		; If no hd, it's still ok.
	move.w	DScreen_hd(Pc),d0	; DScreen handle
	bsr	Free			; Free it
	bsr	crash::uninstall
	jmp	genlib::quit		; Quit the genlib and the program.

DScreen_hd	dc.w	0	; Handle of the DScreen
First_call	dc.w	0	; Premier demarrage ?

	; Include files
	include "sprite.asm"
	include "player.asm"
	include "map.asm"
	include "print.asm"
	include "error.asm"
	include "routines.asm"
	include "search.asm"
	include "game.asm"
	include "crash.asm"
	include "keys.asm"
	include "chests.asm"
	include "object.asm"
	include "position.asm"
	include "member.asm"
	include "tiles.asm"
	include "function.asm"
	debug	<include "test.asm">
	include "event.asm"
	include "script.asm"
	include "dialog.asm"
	include "move.asm"
	include "message.asm"
	include "fight.asm"
	include "console.asm"
	
	include "demos\t3.asm"
	include "demos\introcf.asm"
	include "demos\timedemo.asm"

_comment:
	dc.b	"Chrono Fantasy  "	; Program's name.
	dc.l	VERSION
	dc.b	0
	end			; End of program
