; Module 'Sprite'

; Liste des fonctions :
;	sprite::init
;	sprite::quit
;	sprite::reset
;	sprite::exec
;	sprite::new (Ou sprite::add)
;	sprite::destruct
;	sprite::generate
;	sprite::push
;	sprite::pop
;	sprite::calculate
;	sprite::next

; Initialise la gestion des sprites
; Alloue un buffer entre autre
; In :
;	nothing
; Out :
;	nothing
; Destroy :
;	d0-d2.a0-a1
sprite::init:
	; Allocate the sprite_buffer
	move.w	#(SIZE_LINK_BUFFER+SIZE_UNPRINT_BUFFER+MAX_SPRITE+SIZE_SPRITE_BUFFER+MAX_SPRITE),d0
	bsr	Alloc
	move.w	d0,Sprite_hd
	move.l	a0,Buffer_adr(a6)

; ReInitialise la gestion des sprites
; Met a jour les variables
; In :
;	nothing
; Out :
;	nothing
; Destroy :
;	d0-d2.a0-a1
sprite::reset:
	move.l	Buffer_adr(a6),a0
	adda.w	#SIZE_LINK_BUFFER,a0
	move.l	a0,Sprite_adr(a6)
	move.l	a0,Sprite_end_adr(a6)
	adda.w	#SIZE_UNPRINT_BUFFER+MAX_SPRITE,a0
	move.l	a0,Spr_print_adr(a6)
	move.l	a0,Spr_print_end_adr(a6)
	move.l	a0,Main_adr(a6)
	move.l	a0,Follow_adr(a6)
	clr.w	Spr_level(a6)
	rts


; Ferme le gestionnaire de sprite
; Deaslloue la memoire entre autre
; In :
;	nothing
; Out :
;	nothing
; Destroy :
;	d0-d2.a0-a1
sprite::quit:
	move.w	Sprite_hd(Pc),d0
	bra	Free

Sprite_hd:	dc.w	0




; Recalcule apres un sprite::generate les valeurs de a3/a5
; Attention : Il est imperatif que le sprite soit encore valide
; (Donc pas de destruction du sprite courant !!)
; S'il faut detruire le sprite, utiliser sprite::next
; In:
;	A5.l -> Ancienne adresse du sprite
; Out:
;	A3.l -> Sprite suivant dans le buffer
;	A5.l -> Nouvelle valeur du sprite (Ne change pas normalement !!!)
; Destroy:
;	a3
sprite::calculate:
	move.l	Buffer_adr(a6),a3
\loop:		cmp.l	(a3)+,a5
		bne.s	\loop
	rts


; Detruit le sprite courant, et passe au suivant.
; In:
;	a5.l -> Sprite
;	a3.l -> As usual
; Call:
;	bra	sprite::next
sprite::next:
	move.l	a5,a0
	move.l	4(a3),a5		; Au fait comme si on etait au sprite suivant...
	bsr	sprite::destruct
	bsr.s	sprite::generate
	bsr.s	sprite::calculate	; Ok...
	subq.l	#8,a3
	move.l	(a3)+,a0
	jmp	(a0)


; Sauvegarde l'ensemble des types courants de tous les sprites.
; Attention, il faut tout relier apres ! ! !
; In:
;	Nothing
; Out:
;	Nothing
; Destroy:
;	d0-d2/a0
sprite::push:
	; Il suffit de faire permutter (0,2,3) a Spr_level...
	move.w	Spr_level(a6),d0
	move.w	d0,d1
	bne.s	\no_inc
		addq.w	#1,d0
\no_inc	addq.w	#1,d0
	cmp.w	#4,d0
	beq	error::type_stack_overflow
	move.w	d0,Spr_level(a6)

	; Copie les types...
	move.l	Sprite_adr(a6),a0
	move.l	Sprite_end_adr(a6),d2
	bsr.s	\loop
	move.l	Spr_print_adr(a6),a0
	move.l	Spr_print_end_adr(a6),d2
\loop:		move.b	Type(a0,d1.w),Type(a0,d0.w)
		adda.w	Size(a0),a0
		cmp.l	a0,d2
		bhi.s	\loop
	rts


; Restaure l'ensemble des types courants de tous les sprites.
; Attention, il faut tout relier apres ! ! !
; In:
;	Nothing
; Out:
;	Nothing
; Destroy:
;	d0
sprite::pop:
	; Il suffit de faire permutter (3, 2, 0) a Spr_level...
	move.w	Spr_level(a6),d0
	subq.w	#1,d0
	blt	error::type_stack_overflow
	and.w	#%010,d0
	move.w	d0,Spr_level(a6)
	rts


; Genere les liens entre les sprites.
; Et cree donc le linkage des sprites par une pile (Buffer_adr)
; In:
;	Nothing
; Out:
;	Nothing
; Destroy:
;	Nothing
sprite::generate:
	movem.l	d0-d6/a0-a2,-(a7)

	moveq	#0,d0

	move.w	Spr_level(a6),d1	; Lecture de la profondeur de generation...
	moveq	#1,d6			; Priorite courante
	move.l	Buffer_adr(a6),a1	; Debut...
\while:
		move.w	#250,d5		; d5 = La priorite qui suit celle en court (Initialisee a 250)

		; Premiere pile de sprites (Celle de ceux qui ne s'affichent pas).
		move.l	Sprite_end_adr(a6),d4
		move.l	Sprite_adr(a6),a0
		bsr.s	Spr_loop

		; Meme chose avec la seconde pile de sprites (code identique !)
		move.l	Spr_print_end_adr(a6),d4
		move.l	Spr_print_adr(a6),a0
		bsr.s	Spr_loop
		
		; On a finit avec la priorite courante.
		move.w	d5,d6
		cmp.b	#250,d5
		bcs.s	\while	; On a atteint la priorite finale ?

	; Et on boucle : Rajout du sprite de rebouclage !
	lea	sprite::exec(Pc),a0
	move.l	a0,(a1)+
	move.l	#$01020304,(a1)	; Signature....

	cmp.w	#SIZE_LINK_BUFFER/8-1,d0
	bhi	error::too_many_exe_sprites

	movem.l	(a7)+,d0-d6/a0-a2
	rts

; Sous-routine de sprite::generate
Spr_loop:
\l1:	cmp.l	a0,d4
	bls.s	\end1
		move.b	Prior(a0),d3	; On lit la priorite
		cmp.b	d3,d6		; On la compare a la priorite courante
		bhi.s	\do11		; Elle est inferieure => Sprite suivant
			bne.s	\no1		; Egale, on ajoute !
				clr.w	d2
				move.b	Type(a0,d1.w),d2
				beq.s	\do11
					addq.w	#1,d0
					move.w	Type_table(Pc,d2.w),d2
					lea	Type_table(Pc,d2.w),a2	; Lecture de l'adresse d'execution...
					move.l	a2,(a1)+	; On ecrit l'adresse d'execution et
					move.l	a0,(a1)+	; celles des donnes
					bra.s	\do11
\no1:			cmp.b	d3,d5
			bls.s	\do11
				move.b	d3,d5	; Priorite inferieure a la suivante et superieure a la courante.
\do11:		adda.w	Size(a0),a0	
		bra.s	\l1
\end1:	rts


; Table des types.
; A partir d'un numero de type d2.b, donne l'adresse de la routine a utilisee.
; ADD_TYPE	TYPE_NAME,Type_exec
Type_table:
	ADD_TYPE_VOID	TYPE_nothing
	ADD_TYPE	TYPE_keys,exec_keys
	ADD_TYPE	TYPE_scrolling,exec_scrolling
	ADD_TYPE	TYPE_print,exec_print
	ADD_TYPE	TYPE_syncho,exec_double_buffer
	ADD_TYPE	TYPE_main,exec_main
	ADD_TYPE	TYPE_immobile,exec_immobile
	ADD_TYPE	TYPE_aleatoire,exec_aleatoire
	ADD_TYPE	TYPE_route,exec_route
	ADD_TYPE	TYPE_follow,exec_follow
	ADD_TYPE	TYPE_main_immobile,exec_immobile	; Warning : a corriger !!!!!! Pb si on se teleporte sur un membre de son equipe !
	ADD_TYPE	TYPE_move,exec_move
	ADD_TYPE	TYPE_tiles,exec_tiles
	ADD_TYPE	TYPE_script,exec_script
	ADD_TYPE	TYPE_message,exec_message
 debug	<ADD_TYPE	TYPE_debug,exec_debug>



; Ajoute un sprite a la pile de sprites s'il y a assez de place...
; Pour passer des parametres au module d'initialisation specifique du sprite, on peut utiliser :
;	D3/D4/A4
; In :
;	d0.w = X
;	d1.w = Y
;	d2.w = Type du sprite a initialiser
;	d3/d4 -> Reserve pour la specificite du sprite.
;	a4.l  -> Reserve pour la specificite du sprite.
;	Si player == Vrai 
;		d3.w = No du personnage a initialiser.
;		d4.w = Level du perso ou '0' s'il ne peut pas se battre
;		A4.l -> Dialog struct ou NULL
; Out :	
;	Rien
;	Ajoute le sprite a la table de sprites.
; Destroy :
;	Generalement rien, sauf si la fonction specifique detruit d6/d7/a6
;	ce qui ne doit jamais etre le cas...
; La fonction specifique a pour protocole :
;	SPRITE *special_init(SPRITE *spr=a5, short x=d0, short y=d1)
;	Elle peut modifier tous les registres exeptes : a5-a7/d6-d7
; On definit plusieurs noms, parce que je ne sais jamais quel est le nom exact :)
sprite::new:
sprite::add:
	movem.l	d0-d5/a0-a5,-(a7)
	lsl.w	#3,d2		; D2 = D2 * 8

	; Valeur pour buffer de sprite 'print'
	move.l	Spr_print_adr(a6),d5
	add.l	#(SIZE_SPRITE_BUFFER),d5
	move.l	Spr_print_end_adr(a6),a5

	lea	Sprite_table(PC,d2.w),a3

	tst.b	(a3)
	bne.s	\test
		move.l	Sprite_adr(a6),d5
		add.l	#(SIZE_UNPRINT_BUFFER),d5
		move.l	Sprite_end_adr(a6),a5
\test:

	cmp.l	a5,d5
	bls	error::too_many_sprites

	move.w	2(a3),Type(a5)	; Type & Prio
;	move.b	Type(a5),Type2(a5)
;	move.b	Type(a5),Type3(a5)
	move.w	4(a3),Size(a5)

	tst.b	(a3)
	beq.s	\no_init

		move.w	d0,X(a5)		; Save X
		move.w	d1,Y(a5)		; Save Y
		clr.l	H(a5)			; Set H, Hz, Drawing to 0
		move.b	(a3),Drawing(a5)	; Way of drawing
		move.l	#$01000001,Sens(a5)	; Sens & Ani & Ani_sub & Ani_cpt

		tst.b	1(a3)			; Player ?
		beq.s	\no_init
			bsr	player::init
\no_init:
	; Special init of the sprite
	move.w	6(a3),d5
	jsr	Sprite_table(Pc,d5.w)

	add.w	Size(a5),a5

	tst.b	(a3)
	bne.s	\print
		move.l	a5,Sprite_end_adr(a6)
		bra.s	\done
\print:	move.l	a5,Spr_print_end_adr(a6)
\done
	movem.l	(a7)+,d0-d5/a0-a5
	rts

; Sprites table
; Maximun : 2^(16-4) = 2^12 = 4096 sprites
; ADD_SPR	SPR_NAME,Map_type,Priority,Drawing,Player_flag,spr_size,init_offset
Sprite_table:
 ADD_SPR SPR_main,TYPE_main,105,ANI_BGS_DRAW,0,SPRITE_main,init_main
 ADD_SPR SPR_keys,TYPE_keys,10,0,0,SPRITE_base,Nothing
 ADD_SPR SPR_scrolling,TYPE_scrolling,201,0,0,SPRITE_base,Nothing
 ADD_SPR SPR_print,TYPE_print,220,0,0,SPRITE_base,Nothing
 ADD_SPR SPR_synchro,TYPE_syncho,230,0,0,SPRITE_base,Nothing
; ADD_SPR SPR_decor,TYPE_nothing,110,S16_DRAW,0,SPRITE_print,init_decor
 ADD_SPR SPR_immobile,TYPE_immobile,120,ANI_BGS_DRAW,1,SPRITE_player,Nothing
 ADD_SPR SPR_aleatoire,TYPE_aleatoire,120,ANI_BGS_DRAW,1,SPRITE_alea,init_aleatoire
 ADD_SPR SPR_route,TYPE_route,120,ANI_BGS_DRAW,1,SPRITE_alea,init_route
 ADD_SPR SPR_follow,TYPE_follow,106,ANI_BGS_DRAW,0,SPRITE_follow,init_follow
 ADD_SPR SPR_tiles,TYPE_tiles,205,0,0,SPRITE_data,init_tiles
 ADD_SPR SPR_main_immobile,TYPE_main_immobile,120,ANI_BGS_DRAW,1,SPRITE_player,Nothing
 ADD_SPR SPR_script,TYPE_script,225,0,0,SPRITE_sct,init_script
 ADD_SPR SPR_message,TYPE_message,226,0,0,SPRITE_message,init_message
 debug <ADD_SPR SPR_debug,TYPE_debug,150,0,0,SPRITE_base,init_debug>



; Detruit un sprite et l'enleve de la pile des sprites.
; Attention : apres un appel a cette fonction, il faut regenerer les liens entre
; les sprites (sprite::generate)
; Il est conseille d'attendre de les avoir tous detruits avant d'y faire appel.
; Puis a sprite::calculate si on est dans un sprite.
; In  :
;	a0.l -> Sprite a detruire
;	a5.l -> Sprite courant (ou rien si aucun sprite)
; Out :
;	a5.l -> Sprite courant (mis a jour si necessaire !)
; Destroy :
;	a5
sprite::destruct:
	movem.l	d0-d1/a0-a1,-(a7)

	moveq	#0,d0
	move.w	Size(a0),d0

	move.l	a0,a1
	adda.l	d0,a1		; Destination

	; Selection en fonction de la bonne pile
	cmp.l	Spr_print_adr(a6),a0
	bcc.s	\pspr
		move.l	Sprite_end_adr(a6),d1	; Get spriteADREnd
		sub.l	d0,Sprite_end_adr(a6)	; Correct the new value of SriteADREnd
		bra.s	\done
\pspr	move.l	Spr_print_end_adr(a6),d1	; Get spriteADREnd
	sub.l	d0,Spr_print_end_adr(a6)	; Correct the new value of SriteADREnd
\done:

	; Faut-il corriger a5 ?
	cmp.l	a5,d1
	bls.s	\no_problem
		cmp.l	a0,a5
		bls.s	\no_problem
			suba.l	d0,a5
\no_problem:

	; Deplacement des donnes
	sub.l	a1,d1			; Get number of bytes to move
	beq.s	\end
		lsr.w	#2,d1		; Get number of double words to move
\DelBoucle:		move.l	(a1)+,(a0)+
			dbf	d1,\DelBoucle
\end:	movem.l	(a7)+,d0-d1/a0-a1
	rts




; Execute les sprites en liste chainee
; Attention, un appel a sprite::generate prealablement est indispensable !!
; In :
;	nothing
; Out :
;	nothing
; Destroy :
;	d0-d7/a0-a5
sprite::exec:
	move.l	Buffer_adr(a6),a3
	move.l	(a3)+,a0
	jmp	(a0)


; Inclut les sous-modules contenant le code des sprites
	include "main.asm"
	include "immobile.asm"
	include "aleatoir.asm"
	include "route.asm"
	include "follow.asm"
