; Module 'Event'
; Gestion des differents evenements.

; Fonctions :
;	event::init
;	event::quit
;	event::add
;	event::exe
;	event::is
;	event::deref
;	event::give
;	event::local

; Initialise la gestion des evenements.
; A appeler avant toute utilisation d'une quelconque fonction de ce module
; (Appeler automatiquement lors de la creation d'un jeu, donc en cours du jeu
; c'est inutile)
; In / Out : Nothing
; Destroy: d0-d2/a0-a1
event::init:
	; Alloc memory for table
	move.w	#(MAX_EVENT*2),d0
	bsr	Alloc
	move.w	d0,Story_hd
	move.l	a0,Story_adr(a6)
	; Clear table
	moveq	#MAX_EVENT/2-1,d0
\clr_loop:	clr.l	(a0)+
		dbf	d0,\clr_loop
	; Clear Global PC
	clr.l	Story_pc(a6)
	rts

Story_hd:	dc.w	0



; Libere tout du module 'story'
; A appeler lors de la fin du jeu.
; In / Out / Destroy : nothing
event::quit:
	move.w	Story_hd(Pc),d0
	bra	Free




; Teste si un evenement a deja eu lieu.
; Parcourt la table des histoires a sa recherche.
; Si on rencontre '0', c'est qu'on a atteint la fin de la liste !
; In:
;	d0.w = No de l'evenement
; Out:	d2.b = $00 => Evenement inconnu
;	d2.b = $FF => Evenement deja arrive
; Destroy:
;	d2.b
event::is
	movem.l	d0/d1/d3/a0,-(a7)
	move.l	Story_adr(a6),a0
	clr.b	d2
	move.w	#MAX_EVENT-1,d1
\loop:		move.w	(a0)+,d3
		beq.s	\not_found
		cmp.w	d0,d3
		beq.s	\found
		dbf	d1,\loop
		bra.s	\not_found
\found:
	st.b	d2
\not_found:
	movem.l	(a7)+,d0/d1/d3/a0
	rts



; Ajoute un nouvel evenement.
; N'execute pas le code associe a cet evenement
; N'ajoute pas si le code est deja present
; In  :
;	D0.w = Evenement
; Out : Nothing
; Destroy : nothing
event::add:
	movem.l	d2/a0,-(a7)
	move.l	Story_adr(a6),a0

	; Recherche du premier emplacement libre
	move.w	#MAX_EVENT-1,d2
\loop:		tst.w	(a0)
		beq.s	\found
			cmp.w	(a0)+,d0
			beq.s	\done
		dbf	d2,\loop
		bra	error::too_many_stories
\found:	; Write it
	move.w	d0,(a0)
\done:	movem.l	(a7)+,d2/a0
	rts


; Dereference un evenement pour avoir son code.
; In :
;	d0.w = Evenement
; Out :
;	a0.l -> Code correspondant
; destroy : a0
event::deref:
	move.w	d0,-(a7)
	move.l	ROM_Story_tab(a6),a0
	add.w	d0,d0
	adda.w	0(a0,d0.w),a0
	move.w	(a7)+,d0
	rts


; Donne un evenement.
; Ne le redeclare pas s'il a deja eu lieu.
; Ajoute l'evenement a la table puis
; puis calcule la position du pointeur de lecture de l'evenement global.
; Si on appelle ce sprite, c'est avec un prio < 200 !
; Declenche une erreur s'il y a en un qui tourne
; In:
;	d0.w = Evenement
; Out :
;	Nothing
; Destroy:
;	Nothing
event::give:
	movem.l	d2-d3/a0/a4,-(a7)
	bsr.s	event::is
	tst.b	d2
	blt.s	\done
		tst.l	Story_pc(a6)
		bne	error::privelege
		bsr.s	event::add
		bsr.s	event::deref
		move.l	a0,Story_pc(a6)
		st.b	d3		; Global mode
		suba.l	a4,a4
		moveq	#SPR_script,d2
		bsr	sprite::add
		bsr	sprite::generate
\done:	movem.l	(a7)+,d2-d3/a0/a4
	rts




; Demarre un evenement local.
; Ajoute le script a la pile de sprites et met a jour les liens.
; In:
;	a1.l -> Script Event Code
;	d0.w = X position [Optionnel] -> r0
;	d1.w = Y position [Optionnel] -> r1
;	d2.w = Item no    [Optionnel] -> r2
; Out:
;	Nothing
; Destroy:
;	Nothing
; Warning:
;	Si vous appelez cette routine depuis un sprite,
;	il est imperatif que ce sprite est une priorite < 200 !
;	Ou alors, il faut faire appel a sprite::calculate
event::local
	movem.l	d2-d3/a4,-(a7)
	clr.b	d3		; Local mode
	move.w	d2,-(a7)
	move.w	d1,-(a7)
	move.w	d0,-(a7)
	move.l	a7,a4
	moveq	#SPR_script,d2
	bsr	sprite::add
	addq.l	#6,a7
	bsr	sprite::generate
	movem.l	(a7)+,d2-d3/a4
	rts
