; Module 'game'
; Gestion generale d'un jeu : demarrage, intialisation et finir

; Liste des fonctions :
;	game::init
;	game::destroy
;	game::new
;	game::load
;	game::save
;	game::select
;	game::play
;	game::select_map

; Initalise toutes les variables necessaires avant de commencer un jeu. (Load / New)
; Appel d'abord a destroy_game si un jeu a ete precedemment creer
; Fonction appele par new_game et load_game, donc ;)
; In / Out / destroy : nothing
game::init
	bsr.s	game::destroy

	; Initialisation des differents modules
	bsr	sprite::init
	bsr	print::init
	bsr	dialog::init
	bsr	object::init
	bsr	chest::init
	bsr	member::init
	bsr	event::init
	move.w	#_NOT_FIRST_CALL,First_call	; Check

	; Initalisation des variables.
	clr.l	Save_adr(a6)
	clr.l	Save_name(a6)
	clr.b	Map_name_null(a6)
	clr.l	Map_name(a6)
	st.b	Load_map(a6)

	rts


; Libere toutes les allocations effetues lors de 'init_game' et celles non-deaslloues 
; durant le jeu.
; Fonction appellee par init_game et lors de la fin veritable.
; Donc elle ne doit pas etre appellee autre part !
; Ne doit pas dependre du registre global a6 !
; In / Out : nothing
; Destroy:
;	d0-d2/a0-a1
game::destroy:
	; Securite pour eviter de desallouer n'importe quoi :)
	move.w	First_call(Pc),d0
	cmp.w	#_FIRST_CALL,d0
	beq.s	\quit
		cmp.w	#_NOT_FIRST_CALL,d0
		bne	error::internal		; Erreur grave !!!
			move.w	#_FIRST_CALL,First_call
		
	; Quitte les differents modules :
	bsr	event::quit
	bsr	member::quit
	bsr	chest::quit
	bsr	object::quit
	bsr	dialog::quit
	bsr	print::quit
	bsr	sprite::quit
\quit:	
	; Desallocation des matrices (carte) / Tiles graphiques / liste d'objets...
	jmp	genlib::free_hd		; Ca a ca propre securite

; Charge une nouvelle partie dans le jeu :
;	Creation du fichier de sauvegarde
;	Vide la pile de sprites
;	Initialisation des persos au niveau 1 dans la pile de sprites
;	Initialise l'histoire 0
; In / Out / Destroy : nothing
game::new
	; Demande le fichier de sauvegarde.

	; Creer le fichier dans la VAT
	

	; Demarrage du jeu
	clr.l	Time(a6)
	bsr	game::init
	; moveq	#1,d0
	; bsr	give_story
;	lea	map_name(Pc),a0		
;	moveq	#32,d0
;	moveq	#32,d1	
;	bsr	game::select_map		; Select the current map
	move.b	 #_NORMAL_MODE,Game_mode(a6)	; Select NORMAL

	; Donne quelques persos...
;\w	bra.s	\w
;	moveq	#0,d3
;	bsr	member::new
;	bsr	member::add
;	moveq	#1,d3
;	bsr	member::new
;	bsr	member::add
;	moveq	#2,d3
;	bsr	member::new
;	bsr	member::add

;	bsr	member::save_group
;	move.b	#3,Member_group+Grp_x(a6)
;	move.b	#7,Member_group+Grp_y(a6)

;	bsr	member::basc

;	lea	map2_name(Pc),a0		
;	moveq	#32,d0
;	moveq	#32,d1	
;	bsr.s	game::select_map		; Select the current map
;
;	; Donne quelques persos...
;	moveq	#3,d3
;	bsr	member::new
;	bsr	member::add
;	moveq	#4,d3
;	bsr	member::new
;	bsr	member::add
;	rts

; ReInit sprite
	bsr	sprite::reset
	moveq	#H_START,d0
	bsr	event::give

	; "Falsifie" les tables Height/Graph
	lea	Free_buffer(a6),a0
	move.l	a0,Height_tab(a6)
	move.l	a0,Graph_adr(a6)

	clr.b	Load_map(a6)

	rts


map_name:
	dc.b	"map",0
map2_name:
	dc.b	"map22",0
	even	


; Charge une sauvegarde a partir d'un fichier de sauvegarde
; In :
;	A0 -> File name.
; Out: d0 = 0 -> Succes else fail
; Destroy : ?
game::load
	; Dereference le fichier de sauvegarde

	; Lecture des differentes donnes

	; A0 -> Data
	; Lecture des differents modules
	; A0 est mis a la fin des donnes pour chaque module
	; bsr	load_objects
	; bsr	load_chests
	; bsr	load_story
	; bsr	load_dialog
	rts

; Sauvegarde la partie courante
; (le sprite principal, le nom de la carte courante et la table des objects/evenements)
; In :
;	Nothing
; Out:
;	d0 = 0 -> Succes else fail.
game::save:
	; Dereference le fichier de sauvegarde

	; Sauvegarde des differentes donnes

	; A0 -> Data
	; Sauvegarde des differents modules
	; A0 est mis a la fin des donnes pour chaque module
	; bsr	save_objects
	; bsr	save_chests
	; bsr	save_story
	; bsr	save_dialog

	rts

; Selectionne un jeu dans les sauvegarde courantes
; In : nothing
; Out :
;	a0 -> Filename
;	a1 -> *File ( Skip size & signature)
;	d0.w = Handle
; Destroy : nothing more
game::select
	rts

; Commence a jouer a la partie en cours.
; Charge la carte en cours et execute soit le gestionnaire des cartes, soit le gestionnaire
; d'histoire suivant la varaible 'game_mode'.
; In / Out: nothing
; Destroy : all
game::play:

\loop:
		; Chargement d'une nouvelle carte ?
		tst.b	Load_map(a6)
		beq.s	\no_load
			lea	Map_name(a6),a0
			bsr	map::load
			clr.b	Load_map(a6)	; On ne la recharge plus.
\no_load:

		; Branchement a la bonne routine de jeu
		bsr	sprite::generate
		bsr	sprite::exec
		cmp.b	#_EXIT_MODE,Game_mode(a6)
		bne.s	\loop
	rts


; Selectionne la carte courante et fixe les flags pour la charger prochainement
; si on n'est pas sur la bonne carte.
; Ne change pas le mode courant du jeu
; In:
;	A0 -> Map_name
;	d0.w = X position of main sprite
;	d1.w = Y position of main sprite
; Out:
;	Nothing
; Destroy:
;	nothing
game::select_map:
	movem.l	d2/a0-a2,-(a7)
	; Compare la map...
	lea	Map_name(a6),a1
	move.l	a0,a2
	moveq	#8-1,d2
\cmp_loop	tst.b	(a2)
		beq.s	\fcmp
		tst.b	(a1)
		beq.s	\fcmp
		cmpm.b	(a2)+,(a1)+
		dbne.s	d2,\cmp_loop
	bne.s	\load
\fcmp	cmpm.b	(a2)+,(a1)+
	beq.s	\quit

		; Selectionne la map
\load		lea	Map_name(a6),a1
		moveq	#8-1,d2
\copy_loop:		move.b	(a0)+,(a1)+
			dbf	d2,\copy_loop
		st.b	Load_map(a6)
		; Sauve la position iniale
		move.w	d0,Startx(a6)
		move.w	d1,Starty(a6)
\quit
	movem.l	(a7)+,d2/a0-a2
	rts
