; Module 'Map'
; Gestion de la carte.


; Liste des fonctions
;	map::load
;	map::cmp

; Charge une carte en memoire et met a jour les differentes variables
; In  : A0 -> Map_str
; Out : nothing
; Destroy : all exept a6/a7
map::load:
	; Libere tous les handles precedemments allous sur la precedente carte
	jsr	genlib::free_hd

	; Recherche de la carte dans la VAT
	move.l	#MAP_SIGNATURE,d2
	bsr	search::file2
	addq.l	#(2+4),a0		; Skip size + signature
	move.l	a0,Map_adr(a6)
	
	; A5 -> CMap / A3 -> Plan 1 / A4 -> Plan 2
	move.l	Plan_A_adr(a6),a3
	move.l	Plan_B_adr(a6),a4
	move.l	a0,a5

	; Initialisation de donnes lmentaires.
	moveq	#0,d7			; Clear map time
	move.b	map_size_x(a5),d7	; Read SX1 in d7

	; Initialisation du mode graphique (:->)
	moveq	#0,d0
	move.b	map_print_way(a5),d0
	bsr	print::mode

	; Set the lenght / height of the map
	moveq	#1,d1
	lsl.w	d7,d1
	move.w	d1,Map_x(a6)
	moveq	#0,d0
	move.b	map_size_y(a5),d0
	move.w	d0,Map_y(a6)
	mulu.w	d1,d0
	move.w	d0,Map_size(a6)
	subq.w	#1,d1
	move.w	d1,Mask_x(a6)

	; Mise a jour des coordonnes maximales du scrolling.
	tst.b	Calc(a6)
	bne.s	\92p
		move.w	#(160/2),Center_x(a6)	; 160/2-8
		move.w	#(100/2+16),Center_y(a6) ; 100/2-16
		move.w	#160,d3
		move.w	#100,d4
	bra.s	\done
\92p:		move.w	#(240/2),Center_x(a6)	; 240/2-8
		move.w	#(128/2+16),Center_y(a6)	; 128/2-32
		move.w	#240,d3
		move.w	#128,d4
\done:

	clr.w	xmin(a6)
	clr.w	ymin(a6)
	move.w	Map_x(a6),d0
	lsl.w	#4,d0	; pixel
	sub.w	d3,d0
	move.w	d0,xmax(a6)
	move.w	Map_y(a6),d1
	lsl.w	#4,d1	; pixel
	sub.w	d4,d1
	move.w	d1,ymax(a6)

	; Calculate the coordinates of Plane B
	clr.b	Calc_B(a6)	

	; Recherche et extraction du fichier graphique.
	; A0 -> Gfx_name (Car c'est le 1er de la struct)
	bsr	search::graph_file
	move.l	a0,Gfx1_adr(a6)		; Save it
	move.l	a0,Gfx2_adr(a6)
	move.l	a0,PLANE_TILE_ADR(a3)
	move.l	a0,PLANE_TILE_ADR(a4)
	move.l	a0,genlib::sprite_tile_adr
	move.b	map_size_x(a5),PLANE_SIZE+1(a3)
	move.b	map_size2_x(a5),PLANE_SIZE+1(a4)

	; Calculate the difference between the 2 planes (A & B)
	move.b	map_size2_y(a5),d1
	move.b	map_size_y(a5),d0
	clr.b	d2
	cmp.b	d0,d1
	beq.s	\e_d
	bhi.s	\inv
\l_d:		cmp.b	d0,d1
		bhi.s	\e_d		; Le gros problme est qu'on perd une grosse partie
			add.b	d1,d1		; Du 2eme plan.
			ble.s	\e_d
				addq.b	#1,d2
				bra.s	\l_d
\inv:	cmp.b	d0,d1
	bls.s	\e_d
		lsr.b	#1,d1
		beq.s	\e_d
			subq.b	#1,d2
			bra.s	\inv
\e_d:	move.b	d2,Diff_y(a6)

	; On raffrachit les deux plans
	move.l	a3,a0
	jsr	genlib::refresh_plane
	move.l	a4,a0
	jsr	genlib::refresh_plane

	; Initialisation de shrnklib.
	lea	map_compressed(a5),a0
	jsr	shrnklib::OpenArchive	; open the level-archive
	move.w	d0,Archive_hd(a6)
	beq	error::memory
	jsr	genlib::push_hd

	; Extraction de la matrice du plan A
	moveq	#0,d1			; File 0
	move.l	d1,a0			; Create new
	jsr	shrnklib::Extract
	cmp.l	d1,a0
	beq	error::memory
	move.l	a0,PLANE_TABLE_ADR(a3)
	move.l	a0,Graph_adr(a6)
	move.w	d2,d0
	jsr	genlib::push_hd

	; Extraction de la matrice du plan B
	move.w	Archive_hd(a6),d0
	moveq	#1,d1			; File 1
	suba.l	a0,a0			; Create new
	jsr	shrnklib::Extract
	move.l	a0,d0
	beq	error::memory
	move.l	a0,PLANE_TABLE_ADR(a4)
	move.w	d2,d0
	jsr	genlib::push_hd

	; Extraction de la matrice de hauteur
	move.w	Archive_hd(a6),d0
	moveq	#2,d1			; File 1
	suba.l	a0,a0			; Create new
	jsr	shrnklib::Extract
	move.l	a0,d0
	beq	error::memory
	move.l	a0,Height_tab(a6)
	move.w	d2,d0
	jsr	genlib::push_hd

	; Extraction de la liste des items
	move.w	Archive_hd(a6),d0
	moveq	#3,d1			; File 1
	suba.l	a0,a0			; Create new
	jsr	shrnklib::Extract
	move.l	a0,d0
	beq	error::memory
	addq.l	#4,a0
	move.l	a0,Items_list(a6)
	move.w	d2,d0
	jsr	genlib::push_hd

	; Disparition des coffres deja ouvert
	bsr	chest::remove

	; Initialisation des sprites...
	bsr	sprite::reset

	moveq	#SPR_keys,d2
	bsr	sprite::add		; Sprite : lecture des touches

	bsr	member::sprite		; Tous les Sprites du groupe

	moveq	#SPR_scrolling,d2
	bsr	sprite::add		; Sprite: Gestion scrolling

	moveq	#SPR_print,d2
	bsr	sprite::add		; Sprite : Affichage

	moveq	#SPR_synchro,d2	
	bsr	sprite::add		; Sprite : Synchro

	move.l	Story_pc(a6),d0
	beq.s	\no_global
		move.l	d0,a1
		st.b	d3		; Global mode
		suba.l	a4,a4		; Rien
		moveq	#SPR_script,d2	
		bsr	sprite::add	; Ajoute le script de gestion de l'histoire
\no_global:

	; Initialisation des autres sprites dans l'ecran.
	move.l	Height_tab(a6),a0
	move.l	Items_list(a6),a1
	move.l	a0,a3
	move.w	Map_size(a6),d5
	subq.w	#1,d5
\loop		moveq	#0,d0
		move.b	(a0)+,d0
		cmp.b	#16,d0
		bcs.s	\no_init
			lsr.w	#3,d0	; Get no of items
			subq.w	#1,d0	; Skip the first item
			lsl.w	#3,d0	; Get index of item
			lea	0(a1,d0.w),a2
			tst.b	(a2)+	; Item_type
			bne.s	\no_init
				moveq	#0,d2
				move.b	(a2)+,d2	; Item_sprite
				add.w	d2,d2		; Index in Item_to_sprite_table
				move.w	Item_to_sprite_table(Pc,d2.w),d2	; D2.w = No du sprite
				move.l	a0,d0
				sub.l	a3,d0		; D0.l = Index
				move.w	d0,d1		; D1.l = Index
				and.w	Mask_x(a6),d0	; D0 = Xcase
				lsl.w	#4,d0
				addq.w	#8,d0		; D0 = X = 16*Xcase + 8
				lsr.w	d7,d1
				addq.w	#1,d1
				lsl.w	#4,d1		; D1 = Y = 16*Ycase + 16
				move.w	(a2)+,d3	; D3 = No du perso
				move.w	(a2)+,d4	; D4 = Level
				movea.w	(a2)+,a4
				adda.l	a1,a4		; A4 -> Dialog
				bsr	sprite::add
\no_init:	dbf	d5,\loop

	; Fin de l'initialisation des autres sprites

	; Initialisation du gestionnaire de tiles...
	move.l	Items_list(a6),a4
	move.w	-4(a4),d0
	beq.s	\no_tiles
		moveq	#SPR_tiles,d2
		adda.w	d0,a4
		bsr	sprite::add
\no_tiles:

	; Initialisation du script de la map...
	move.l	Items_list(a6),a1
	move.w	-2(a1),d0
	beq.s	\no_script
		clr.b	d3			; Local
		adda.w	d0,a1			; a1 -> code
		suba.l	a4,a4			; a4 -> Nothing...
		moveq	#SPR_script,d2
		bsr	sprite::add
\no_script:
;	lea	Boite(Pc),a4
;	moveq	#0,d4
;	moveq	#127,d3
;	moveq	#SPR_message,d2
;	bsr	sprite::add
	rts

Item_to_sprite_table:
	dc.w	SPR_immobile
	dc.w	SPR_aleatoire
	dc.w	SPR_route



; Compare le nom de deux cartes
; In:
;	a1.l -> 1ere carte
;	a2.l -> 2eme carte
; Out:
;	d0.b = 0 => Ok
;	d0.b = $Ff => Different
; Destroy :
;	d0
map::cmp:
	movem.l	a1-a2/d2,-(a7)
	st.b	d0
	moveq	#5-1,d2
\cmp_loop	tst.b	(a2)
		beq.s	\fcmp
		tst.b	(a1)
		beq.s	\fcmp
		cmpm.b	(a2)+,(a1)+
		dbne.s	d2,\cmp_loop
	bne.s	\done
\fcmp	cmpm.b	(a1)+,(a2)+
	bne.s	\done
		clr.b	d0
\done:	movem.l	(a7)+,a1-a2/d2
	rts

;init_decor:
;	move.b	d3,Ani(a5)
;	move.b	#3,H(a5)
;	move.w	#3*32,Hz(a5)
;	rts
	