; Fichier 'member'
; Gestionnaire des membres de l'equipe.

; Liste des fonctions
;	member::init
;	member::quit
;	member::is
;	member::new
;	member::add
;	member::deref
;	member::remove
;	member::exchange
;	member::sprite
;	member::center
;	member::next
;	member::save_group
;	member::basc
;	member::reset
;	member::load
;	member::save
;	member::calculate


; Inialise le gestionnaire de membres.
; In :
;	nothing
; Out :
;	nothing
; Destroy:
;	d0-d2/a0-a1
member::init:
	clr.w	Member_hd
	clr.l	Member_adr(a6)
	clr.w	Member_size(a6)
	clr.w	Member_cpt(a6)
	lea	Member_group(a6),a0
	moveq	#10*3-1,d0
\loop:		clr.b	(a0)+
		dbf	d0,\loop
	rts

; Quitte le gestionnaire de membres.
; (Desalloue la memoire...)
; In :
;	nothing
; Out :
;	nothing
; Destroy:
;	d0-d2/a0-a1
member::quit:
	move.w	Member_hd(Pc),d0
	bra	Free

Member_hd	dc.w	0



; Creer un nouveau membre dans le groupe.
; Attribue un numero de membre.
; Demande le nom, mais ne l'ajoute pas dans le groupe courant.
; In:
;	d3.w = Perso
; Out:
;	nothing
; Destroy:
;	nothing
member::new:
	movem.l	d0-d4/a0-a2,-(a7)

	; ReAlloue le tableau...
	moveq	#0,d0
	move.w	Member_size(a6),d0
	move.l	d0,d4
	add.w	#plf_struct,d0
	move.w	d0,Member_size(a6)
	move.l	d0,-(a7)
	move.w	Member_hd(Pc),-(a7)
	jsr	tios::HeapRealloc
	addq.l	#6,a7
	move.w	d0,Member_hd
	beq	error::memory
		
	; Ajoute les donnes
	tios::DEREF	d0,a1
	move.l	a1,Member_adr(a6)
	adda.l	d4,a1

	suba.l	a2,a2
	moveq	#1,d4
	bsr	player::init_fight_data

	; Et un de plus !
	addq.w	#1,Member_cpt(a6)

	movem.l	(a7)+,d0-d4/a0-a2
	rts




; Dereference un numero de personnage en son numero de groupe
; et son pointeur vers la structure Fight_player correspondante
; In :
;	d3.w = Perso
; Out:
;	d0.w = Numero de membre
;	a0.l -> Player_fight struct
; Destroy:
;	d0/a0
member::deref
	move.l	Member_adr(a6),a0

	move.w	Member_cpt(a6),d0
	subq.w	#1,d0
	blt.s	\error
\loop:		cmp.w	plf_no(a0),d3
		beq.s	\found
		lea	plf_struct(a0),a0
		dbf	d0,\loop
\error:	bra	error::player_no_member
\found:	sub.w	Member_cpt(a6),d0
	neg.w	d0
	rts



; Ajoute un membre au groupe courant
; Attention : il doit etre au prealable creer !
; S'il y est deja, ne fait rien.
; In:
;	d3.w = Perso
; Out:
;	nothing
; Destroy:
;	nothing
member::add:
	movem.l	d0-d1/a0-a2,-(a7)

	; Recherche du numero de membre
	bsr.s	member::deref

	; Verifie s'il y est deja...
	cmp.w	Member_group+Grp_nbr(a6),d0
	bls.s	\done		; On ajoute pas ce qui est deja !
		; On recopie temporairement...
		lea	Free_buffer(a6),a1
		moveq	#plf_struct-1,d1
\cpy:			move.b	(a0)+,(a1)+
			dbf	d1,\cpy
		; On calcule ou on va recopier
		move.l	Member_adr(a6),a0
		move.l	a0,a2
		move.w	Member_group+Grp_nbr(a6),d0
		mulu.w	#plf_struct,d0
		adda.l	d0,a0			; Ou on veut le mettre
		; On decale
		move.w	Member_size(a6),d1
		sub.w	#plf_struct,d1
		adda.w	d1,a2
		sub.w	d0,d1
		subq.w	#1,d1
		blt.s	\no_decal
\cpy2:			move.b	-(a2),plf_struct(a2)
			dbf	d1,\cpy2
\no_decal:
		; Et on recopie a partir de la copie
		lea	Free_buffer(a6),a1
		moveq	#plf_struct-1,d1
\cpy3:			move.b	(a1)+,(a0)+
			dbf	d1,\cpy3
		; Et on ajoute un membre
		addq.w	#1,Member_group+Grp_nbr(a6)
\done:
	movem.l	(a7)+,d0-d1/a0-a2
	rts

; Enleve un membre du groupe courant
; S'il est deja enlever (vaquant ou dans un autre group), ne fait rien
; In:
;	d3.w = Perso
; Out:
;	nothing
; Destroy:
;	nothing
; Pas au point !!!!
member::remove:
	movem.l	d0-d1/a0-a2,-(a7)

	; Recherche du numero de membre
	bsr	player::deref

	; Verifie s'il y est deja...
	cmp.w	Member_group+Grp_nbr(a6),d0
	bhi.s	\done		; On enleve pas ce qu'il n'y a pas !
		; On recopie temporairement...
		lea	Free_buffer(a6),a1
		moveq	#plf_struct-1,d1
\cpy:			move.b	(a0)+,(a1)+
			dbf	d1,\cpy
		; On calcule ou on va recopier
		move.l	Member_adr(a6),a0
		move.w	Member_group+Grp_nbr(a6),d0
		mulu.w	#plf_struct,d0
		move.w	Member_size(a6),d1
		adda.w	d1,a0
		move.l	a0,a2
		; On decale
		sub.w	#plf_struct,d1
		sub.w	d0,d1
		subq.w	#1,d1
		blt.s	\no_decal
\cpy2:			move.b	-(a2),-plf_struct(a2)
			dbf	d1,\cpy2
\no_decal:
		; Et on recopie a partir de la copie
		lea	Free_buffer(a6),a1
		moveq	#plf_struct-1,d1
\cpy3:			move.b	(a1)+,(a0)+
			dbf	d1,\cpy3
		; Et on enleve un membre
		subq.w	#1,Member_group+Grp_nbr(a6)
\done:
	movem.l	(a7)+,d0-d1/a0-a2
	rts


; Echange les deux personnages.
; In:
;	d0.w = Player1
;	d1.w = Player2
; Out:
;	nothing
; Destroy:
;	nothing
member::exchange
	movem.l	d0-d3/a0-a1,-(a7)

	move.w	d0,d3
	bsr	member::deref

	; Recopie dans tampon de 1
	lea	Free_buffer(a6),a1
	moveq	#plf_struct-1,d0
\cpy:		move.b	(a0)+,(a1)+
		dbf	d0,\cpy

	; Recopie dans tampon de 2
	lea	-plf_struct(a0),a1
	move.w	d1,d3
	bsr	member::deref
	moveq	#plf_struct-1,d0
\cpy2:		move.b	(a0)+,(a1)+
		dbf	d0,\cpy2

	; Derniere copie
	lea	Free_buffer+plf_struct(a6),a1
	moveq	#plf_struct-1,d0
\cpy3:		move.b	-(a1),-(a0)
		dbf	d0,\cpy3
	
	movem.l	(a7)+,d0-d3/a0-a1
	rts

; Teste si un perso fait parti du groupe courant.
;In:
;	d3.w = No de perso
;Out:
;	d2.b = 00 => Le perso ne fait pas parti du groupe courant
;	d2.b != 0 => C'est OK
; Destroy:
;	d2
member::is:
	movem.l	a0/d0,-(a7)
	move.l	Member_adr(a6),a0
	move.w	Member_group+Grp_nbr(a6),d0
	st.b	d2
	subq.w	#1,d0
\loop		cmp.w	plf_no(a0),d2
		beq.s	\found
		lea	plf_struct(a0),a0
		dbf	d0,\loop
	clr.b	d2
\found	movem.l	(a7)+,d0/a0
	rts



; Initialise tous les sprites_player necessaire a la gestion des membres.
; A savoir : main / follow / fixe
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	d0-d4/a0-a4
member::sprite:
	; Initialise main
	move.l	Member_adr(a6),a0
	move.w	plf_no(a0),d3
	move.w	plf_level(a0),d4
	suba.l	a4,a4
	move.l	Spr_print_end_adr(a6),plf_adr(a0)	; Pas terrible :(
	moveq	#SPR_main,d2
	bsr	sprite::add

	; Initialise les suivants.
	move.w	Member_group+Grp_nbr(a6),d0
	subq.w	#2,d0
	blt.s	\done_follow
\loop_follow:	
		lea	plf_struct(a0),a0
		move.w	plf_no(a0),d3
		move.w	plf_level(a0),d4
		suba.l	a4,a4
		move.l	Spr_print_end_adr(a6),plf_adr(a0)	; Pas terrible :(
		moveq	#SPR_follow,d2
		bsr	sprite::add
		dbf	d0,\loop_follow
\done_follow:

	; Inialise si meme carte les autres groupes !
	lea	plf_struct(a0),a0
	
	lea	Map_name(a6),a2

	; Groupe 2 : compare les cartes et verifie le nombre de personne
	tst.w	Member_group+Group_struct+Grp_nbr(a6)
	beq.s	\groupe3

	lea	Member_group+Group_struct+Grp_map(a6),a1
	bsr	map::cmp
	tst.b	d0
	bne.s	\groupe3
		; X & Y
		clr.w	d0
		clr.w	d1
		move.b	Member_group+Group_struct+Grp_x(a6),d0
		move.b	Member_group+Group_struct+Grp_y(a6),d1
		lsl.w	#4,d0
		lsl.w	#4,d1
		; No & level & dialog
		move.w	plf_no(a0),d3
		moveq	#0,d4
		lea	Change_party_member_str(Pc),a4
		move.l	Spr_print_end_adr(a6),plf_adr(a0)
		moveq	#SPR_main_immobile,d2
		bsr	sprite::add
\groupe3:

	move.w	Member_group+Group_struct+Grp_nbr(a6),d0
	mulu.w	#plf_struct,d0
	adda.l	d0,a0

	; Groupe 3 : compare les cartes et verifie le nombre de personne
	tst.w	Member_group+Group_struct*2+Grp_nbr(a6)
	beq.s	\groupe4
	lea	Member_group+Group_struct*2+Grp_map(a6),a1
	bsr	map::cmp
	tst.b	d0
	bne.s	\groupe4
		; X & Y
		clr.w	d0
		clr.w	d1
		move.b	Member_group+Group_struct*2+Grp_x(a6),d0
		move.b	Member_group+Group_struct*2+Grp_y(a6),d1
		lsl.w	#4,d0
		lsl.w	#4,d1
		; No & level & dialog
		move.w	plf_no(a0),d3
		moveq	#0,d4
		lea	Change_party_member_str(Pc),a4
		move.l	Spr_print_end_adr(a6),plf_adr(a0)
		moveq	#SPR_main_immobile,d2
		bsr	sprite::add
\groupe4:

	rts


Change_party_member_str:
	dc.b		SetDialog
	dc.b		PutBox,3
	dc.b		ChangeFont,Medium
	dc.b		ChangeColor,Black
	dc.b		"Change party members ?"
	dc.b		WaitKey
	dc.b		EndDialog
	EVEN





; Recentre tous les personnages autour du personnage principal
; A savoir : main / follow
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	nothing
member::center:
	movem.l	d0-d1/a0-a1,-(a7)
	move.l	Main_adr(a6),a0
	move.w	X(a5),d0
	move.w	Y(a5),d1

	; Inialise les suivants.
	move.l	Spr_print_adr(a6),a0
\loop:		cmp.b	#TYPE_follow,Type(a0)
		bne.s	\next
			clr.w	Tampon_ptr(a0)
			lea	Tampon(a0),a1
			moveq	#8-1,d2
\loop2				move.w	d0,(a1)+
				move.w	d1,(a1)+
				clr.w	(a1)+
				dbf	d2,\loop2
			move.w	d0,X(a0)
			move.w	d1,Y(a0)
\next:		adda.w	Size(a0),a0
		cmp.l	Spr_print_end_adr(a6),a0
		bmi.s	\loop

	movem.l	(a7)+,d0-d1/a0-a1
	rts


; Sauvegarde les donnes du groupe courant.
; (Sa position, le nom de la carte)
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	nothing
member::save_group:
	movem.l	d0/a0,-(a7)
	move.l	Main_adr(a6),a0
	move.w	X(a0),d0
	lsr.w	#4,d0
	move.b	d0,Member_group+Grp_x(a6)
	move.w	Y(a0),d0
	lsr.w	#4,d0
	move.b	d0,Member_group+Grp_y(a6)
	
	move.l	Map_name(a6),Member_group+Grp_map(a6)
	move.w	Map_name+4(a6),Member_group+Grp_map+4(a6)

	movem.l	(a7)+,d0/a0
	rts

; Remet a zero le compteur de chque groupe
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	nothing
member::reset:
	clr.w	Member_group+Grp_nbr(a6)
	clr.w	Member_group+Group_struct*1+Grp_nbr(a6)
	clr.w	Member_group+Group_struct*2+Grp_nbr(a6)
	rts

; Bascule les groupes...
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	Nothing
member::basc
	movem.l	d0-d1/a1-a2,-(a7)

	; Permute les personnages
	; Sauve les premiers dans le buffer...
	move.l	Member_adr(a6),a1
	lea	Free_buffer(a6),a2
	move.w	Member_group+Grp_nbr(a6),d0
	mulu.w	#plf_struct,d0
	move.w	d0,d1
	subq.w	#1,d0
	blt.s	\no
\loop:		move.b	(a1)+,(a2)+
		dbf	d0,\loop
\no:
	; Fait decaler les autres.
	move.l	Member_adr(a6),a2
	move.w	Member_group+Group_struct+Grp_nbr(a6),d0
	add.w	Member_group+Group_struct*2+Grp_nbr(a6),d0
	mulu.w	#plf_struct,d0
	subq.w	#1,d0
	blt.s	\no2
\loop2:		move.b	(a1)+,(a2)+
		dbf	d0,\loop2
\no2

	; Et recopie le premier apres...
	lea	Free_buffer(a6),a1
	subq.w	#1,d1
	blt.s	\no3
\loop3:		move.b	(a1)+,(a2)+
		dbf	d1,\loop3
\no3:

	; Fait basculer les groupes.
	lea	Member_group(a6),a1
	move.l	a1,a2
	move.l	(a1)+,-(a7)
	move.l	(a1)+,-(a7)
	move.w	(a1)+,-(a7)

	; Recopie le 2 dans le 1
	move.l	(a1)+,(a2)+
	move.l	(a1)+,(a2)+
	move.w	(a1)+,(a2)+

	; Recopie le 3 dans le 2
	move.l	(a1)+,(a2)+
	move.l	(a1)+,(a2)+
	move.w	(a1)+,(a2)+

	; Recopie le 1 dans le 3
	move.w	(a7)+,4+4(a2)
	move.l	(a7)+,4(a2)
	move.l	(a7)+,(a2)

	movem.l	(a7)+,d0-d1/a1-a2
	rts



; Sauvegarde les donnes du groupe courant.
; Puis passe au suivant.
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	a1/a2/d0-d2
member::next:
	; Sauvegarde les donnes courantes dans le groupe
	bsr	member::save_group

	; Bascule les groupes
	moveq	#0,d2
\basc:
	bsr.s	member::basc
	addq.w	#1,d2
	tst.w	Member_group+Grp_nbr(a6)
	beq.s	\basc

	subq.w	#3,d2	; S'il n'y a qu'un groupe
	bne.s	\ok	; On laisse tomber !
		rts
\ok:
	; Detruit les sprites de Type Follow et de Type main...
;	move.l	Spr_print_adr(a6),a1
;\loop:		move.b	Type(a1),d0;
;		cmp.b	#TYPE_main,d0
;		beq.s	\ok
;			cmp.b	#TYPE_follow,d0
;			beq.s	\ok
;				cmp.b	#TYPE_main_immobile,d0
;				bne.s	\next
;\ok:					movem.l	d0-d7/a0-a6,-(a7)
;					move.l	a1,a5
;					bsr	sprite::destruct
;					movem.l	(a7)+,d0-d7/a0-a6
;					bra.s	\no_next
;\next:		adda.w	Size(a1),a1
;\no_next:	cmp.l	Spr_print_end_adr(a6),a1
;		bmi.s	\loop
	
	; Inialise les persos...
;	bsr	member::sprite

	; Charge la carte
	clr.b	Map_name(a6)
	lea	Member_group+Grp_map(a6),a1
	bra	Item_map	; Bonne position de la pile !
				; Doit imperativement quitter la boucle de liaisons !


; Sauvegarde les donnes
; In:
;	a0 -> Current data in save file
; Out:
;	a0 -> Fin des donnes
; Destroy:
;	d0-d2/a0-a2
member::save:
	bsr	member::save_group

	move.w	Member_size(a6),(a0)+
	move.w	Member_cpt(a6),(a0)+

	lea	Member_group(a6),a1
	moveq	#3*Group_struct/2-1,d0
\grp_loop:
		move.w	(a1)+,(a0)+
		dbf	d0,\grp_loop

	move.l	Member_adr(a6),a1
	move.w	Member_size(a6),d0
	subq.w	#1,d0
\loop:		move.b	(a1)+,(a0)+
		dbf	d0,\loop
	rts


; Calcule la taille du fichier a sauvegarder
; In : 
;	nothing
; Out:
;	d0.w = Taille a allouer
; Destroy:
;	d0-d2/a0-a1
member::calculate:
	move.w	Member_size(a6),d0
	add.w	#2+2+3*Group_struct,d0
	rts


; Charge les donnes
; In:
;	a0 -> Current data in save file
; Out:
;	a0 -> Fin des donnes
; Destroy:
;	d0-d2/a0-a2
member::load:
	move.w	(a0)+,d2
	move.w	d2,Member_size(a6)
	move.w	(a0)+,Member_cpt(a6)
	
	lea	Member_group(a6),a1
	moveq	#3*Group_struct/2-1,d0
\grp_loop:
		move.w	(a0)+,(a1)+
		dbf	d0,\grp_loop

	; Alloue
	move.w	d2,d0
	move.l	a0,a1
	bsr	Alloc
	exg	a0,a1
	move.w	d0,Member_hd
	beq	error::memory

	move.l	a1,Member_adr(a6)
	move.w	Member_size(a6),d0
	subq.w	#1,d0
\loop:		move.b	(a0)+,(a1)+
		dbf	d0,\loop
	rts
	