; Module player

; Gestion interne des personnages
; Entre autre, ce module fait surtout de l'initialisation et la mise au point
; des valeurs.

; Liste des fonctions :
;	player::init
;	player::deref
;	player::fight_data
;	player::check_pos
;	player::get_block
;	player::set_block
;	player::set_item
;	player::move
;	player::animation
;	player::deref_2_stack
;	player::calc_sens




; Cacule le sens a donne aux personnages suivant les coordonnes
; relatives de sa direction
; Attention, la direction n'a pas pour origine le personnage
; (C'est etrange, je sais !)
; In :
;	d0.w = Dx = (-Direction_X)
;	d1.w = Dy = (-Direction_Y)
; Out:
;	d2.w = Sens
; Destroy :
;	d0-d2
; Exemple:
;	d0.w = X - X_perso / d0.w = X- X_dest
player::calc_sens:
	cmp.w	d1,d0			; X > Y ?
	blt.s	\bas_droite
\haut_gauche:
		neg.w	d0
		cmp.w	d1,d0			; Y < -X ?
		bgt.s	\haut
\gauche:		moveq	#2,d2		; Gauche
			bra.s	\end
\haut:		moveq	#1,d2			; Bas
		bra.s	\end
\bas_droite:
	neg.w	d0
	cmp.w	d1,d0
	bgt.s	\droite
\bas:		moveq	#0,d2		; Haut
		bra.s	\end
\droite:
	moveq	#3,d2
\end:	rts





; Initialise le personnage dans la pile de sprite.
; Attention : cette routine ne modifie pas la valeur de X & de Y
; Met a jour Hz & H suivant la hauteur de la case
; In:
;	A5.l -> Player struct a initialiser : doit etre assez grand !
;		i.e. si LEVEL = 0, struct du type mplayer, sinon fplayer
;	d3.w = No du personnage a initialiser.
;	d4.w = Level du perso ou '0' s'il ne peut pas se battre
;	A4.l -> Dialog struct ou NULL
; Out:
;	rien
; Destroy:
;	nothing
player::init:
	movem.l	d0-d4/a0-a4,-(a7)

	; Initialisation
	move.w	d3,Player_no(a5)
	bsr.s	player::deref

	; Sauvegarde la structure de dialogue du perso
	move.l	a4,Diag_adr(a5)

	; Init animation
	move.w	ROM_gfx_table(a0),d0
	lea	0(a0,d0.w),a2			; A2 = *Gfx
	move.l	a2,Gfx_adr(a5)
	move.w	ROM_ani_table(a0),d0
	lea	0(a0,d0.w),a2
	move.l	a2,Ani_adr(a5)			; A2 = *Ani

	; Init state
	clr.b	H(a5)

	movem.l	(a7)+,d0-d4/a0-a4
	rts

; A partir du numero d'un personnage, determine l'adresse ROM du perso.
; Si c'est un numero negatif, dereference a partir des membres
; de l'equipe courante
; In:
;	d3.w = Perso 
; Out:
;	A0.l -> Adresse ROM du perso ou NULL
; Destroy:
;	a0
player::deref:
	move.l	d3,-(a7)
	suba.l	a0,a0
	cmp.w	#MAX_PLAYER-1,d3
	bhi.s	\found
		mulu	#Player_rom_size,d3
		move.l	ROM_Player_tab(a6),a0
		adda.l	d3,a0
\found:	move.l	(a7)+,d3
	rts



; Initialise la structure 'combat' du personnage
; Attention : Reinialise tout !
; In:
;	A1.l -> Fight Player struct a initialiser
;	A2.l -> Sprite player associe
;	d3.w =  No du personnage a initialiser.
;	d4.w =  Level du perso ou '0' s'il ne peut pas se battre
; Out:
;	d0.b = 0 => Ok
;	d0.b = 1 => Le perso n'a pas de structure statut !
; Destroy:
;	d1
player::init_fight_data:
	movem.l	d1-d3/a0-a1,-(a7)
	bsr.s	player::deref

	moveq	#1,d0
	move.w	ROM_statut(a0),d1
	beq.s	\error
		
		adda.w	d1,a0
		move.w	d3,plf_no(a1)
		move.l	a2,plf_adr(a1)
		move.w	d4,plf_level(a1)

		moveq	#0,d3
		move.w	ST_EXP(a0),d3
		move.l	d3,plf_exp(a1)
		move.b	ST_texp(a0),d3
		lsl.w	#4,d3
		move.w	d3,plf_texp(a1)

		move.l	ST_HP(a0),plf_HP(a1)
		move.w	ST_MP(a0),plf_MP(a1)
	
		bsr.s	player::reinit_fight_data

		clr.w	d0
\error:
	movem.l	(a7)+,d1-d3/a0-a1
	rts

; ReInitialise la structure 'combat' du personnage
; Attention : Ne reinialise pas
;	+HP/MP/Weapon/Armor/Helmet/Accessory
;	+Level/exp/texp/no/adr
; In:
;	A1.l -> Fight Player struct a initialiser
; Out:
;	nothing
; Destroy:
;	nothing
player::reinit_fight_data:
	movem.l	d0-d4/a0-a4,-(a7)
	move.w	plf_no(a1),d3
	bsr.s	player::deref

	move.w	ROM_statut(a0),d1
	beq.s	\error
		
		adda.w	d1,a0

		move.l	ST_HP(a0),plf_HP_MAX(a1)
		move.w	ST_MP(a0),plf_MP_MAX(a1)
	
		move.l	ST_phy_att(a0),plf_phy_att(a1)	; Phy / Feu
		move.l	ST_eau_att(a0),plf_eau_att(a1)	; Eau / Lumiere
		move.l	ST_omb_att(a0),plf_omb_att(a1)	; Ombre / Esprit

		move.w	ST_speed(a0),plf_speed(a1)	; Speed & Luck
		clr.w	plf_wait(a1)			; Wait & Limit
		
		; Ajuste les donnes grace au niveau courant...
		; A faire...

\error:
	movem.l	(a7)+,d0-d4/a0-a4
	rts



; Met a jour l'animation du personnage
; Si ce n'est pas l'animation en cours du perso, on effectue un reset des compteurs.
; In :
;	A5   = *Player
;	D5.b = Nouvelle (?) animation
; Out :
;	Met a jour dans la structure la nouvelle animation
; Destroy:
;	Rien
player::animation:
	cmp.b	Ani(a5),d5
	beq.s	\Ok
		move.b	d5,Ani(a5)
		move.b	#1,Ani_cpt(a5)
\Ok	rts




; Lit le bloc de collision et l'item de la case (X,Y).
; In
;	d0.w = X
;	d1.w = Y
; Out:
;	d0.w = State
;	d1.w = No of item in the item-list
player::get_block:
	move.l	a1,-(a7)
	move.l	Height_tab(a6),a1
	lsr.w	#4,d1			; Pixel -> Case
	lsr.w	#4,d0			; Pixel -> Case
	lsl.w	d7,d1
	add.w	d1,d0			; D0.w = E(X/16) + 2^TailleX * E(Y/16)
	moveq	#0,d1
	move.b	0(a1,d0.w),d1		; D1.w = Type de la case
	move.w	d1,d0
	andi.w	#7,d0			; D0.w = Bloc
	lsr.w	#3,d1			; D1.w = Item
	move.l	(a7)+,a1
	rts

; Ecrit le bloc de collision de la case (X,Y).
; In
;	d0.w = X
;	d1.w = Y
;	d2.w = Block
; Out: nothing
player::set_block:
	move.l	a1,-(a7)

	; Calcul adresse
	move.l	Height_tab(a6),a1
	lsr.w	#4,d1			; Pixel -> Case
	lsr.w	#4,d0			; Pixel -> Case
	lsl.w	d7,d1
	add.w	d1,d0			; D0.w = E(X/16) + 2^TailleX * E(Y/16)
	adda.w	d0,a1

	; Traitement
	andi.b	#$F8,(a1)
	or.b	d2,(a1)

	move.l	(a7)+,a1
	rts

; Ecrit l'item de la case (X,Y).
; In
;	d0.w = X
;	d1.w = Y
;	d2.w = Item
; Out: nothing
player::set_item:
	move.l	a1,-(a7)

	; Calcul adresse
	move.l	Height_tab(a6),a1
	lsr.w	#4,d1			; Pixel -> Case
	lsr.w	#4,d0			; Pixel -> Case
	lsl.w	d7,d1
	add.w	d1,d0			; D0.w = E(X/16) + 2^TailleX * E(Y/16)
	adda.w	d0,a1

	; Traitement
	andi.b	#$03,(a1)
	lsl.w	#3,d3
	or.b	d3,(a1)

	move.l	(a7)+,a1
	rts

; Verifie si le personnage peut etre aux coordonnes (x,y)
; (Compare les hauteurs).
; In :
;	A5   = *Player
;	D0.w = X
;	D1.w = Y
; Out:
;	d0.b = BOOL = '+/-1' si c Ok sinon c '0'
;	d1.w = Etat suivant (ou -1)
;	a1   = *Height_tab
;	Flags sont mis a jour suivant la valeur de d0.b (Comme si on avait fait un tst.b d0)
player::check_pos:
	move.l	Height_tab(a6),a1
check_pos_internal:
	lsr.w	#4,d1			; Pixel -> Case
	lsr.w	#4,d0			; Pixel -> Case
	lsl.w	d7,d1
	add.w	d1,d0			; D0.w = E(X/16) + 2^TailleX * E(Y/16)
	moveq	#0,d1
	move.b	0(a1,d0.w),d1		; D1.w = Type de la case
	andi.w	#7,d1				; On filtre le type
	add.b	H(a5),d1			; D0.w = Etat courant
	move.b	check_state_tab(Pc,d1.w),d1
	blt.s	\pas_pos
		move.b	d1,H(a5)
		cmp.b	#24,d1
		bls.s	\avant
			andi.b	#%01111111,Drawing(a5)
			moveq	#1,d0
			rts
\avant:		ori.b	#%10000000,Drawing(a5)
			moveq	#1,d0
			rts
\pas_pos:
			clr.b	d0
			rts
check_state_tab:
	;	0a 0b 1a 1b 02 03 0/ 0\
	dc.b	08,32,16,40,48,-1,08,08			; Etat inter = 00
	dc.b	08,32,16,-1,24,-1,08,08			; Etat : 0.a = 08
	dc.b	08,-1,16,-1,24,-1,16,16			; Etat : 1.a = 16
	dc.b	08,-1,16,-1,24,-1,24,24			; Etat : 2.a = 24
	dc.b	08,32,-1,40,48,-1,32,32			; Etat : 0.b = 32
	dc.b	-1,32,-1,40,48,-1,40,40			; Etat : 1.b = 40
	dc.b	-1,32,-1,40,48,-1,48,48			; Etat : 2.b = 48


; Valide un deplacement : enregistre, si possible, les coordonnes (x,y) dans le sprite
; In:
;	A5   = *Sprite Player
;	D3.w = X
;	D4.w = Y
; Out:
;	A5   = *Sprite Player
;	A1   = *Height_tab
;	D2.w =	0 si c'est ok
;		1 si c'est possible seulement sur les y
;		2 si c'est possible seulement sur les x
;		3 si c'est impossible
; Detruit :
;	d0/d1
player::check_move:
	moveq	#3,d2
	move.l	Height_tab(a6),a1
	; X
	move.w	d3,d0
	move.w	Y(a5),d1
	bsr	check_pos_internal
	beq.s	\Pas_okx
		move.w	d3,X(a5)
		moveq	#2,d2
\Pas_okx:
	; Y
	move.w	X(a5),d0
	move.w	d4,d1
	bsr	check_pos_internal
	beq.s	\Pas_oky
		move.w	d4,Y(a5)
		subq.w	#2,d2
\Pas_oky:
	rts


; Determine les nouvelles coordonnes d'un sprite.
; En fait, deplace le sprite en direction de la destination
; In :
;	A5 -> Player sprite
; Out : Met a jour les coordonnnes
; Destroy : d0, d1, d2, d3 & d4 & a1
player::move:
	; Calcul de direction
	move.w	X(a5),d0
	move.w	Y(a5),d1

	move.w	d0,d3
	move.w	d1,d4

	moveq	#0,d2		; Direction d2 = 0
	
	sub.w	Xd(a5),d0
	bhi.s	\ok
		neg.w	d0	; D0 >0
		moveq	#8*2,d2	; X < Dest_X
\ok:
	sub.w	Yd(a5),d1
	bhi.s	\ok2
		neg.w	d1	; D1 > 0
		addq.w	#4*2,d2	; Y < Dest_Y
\ok2:
	cmp.w	d1,d0
	bhi.s	\ok3
		addq.w	#2*2,d2	; Dx < Dy
\ok3:
	; lecture de Inc_x et de Inc_y
	move.l	Move_player_table(Pc,d2.w),d2	; D2.l = Inc_x.w | Inc_y.w

	; D2 = Correct direction
	cmp.w	#3,d1
	bhi.s	\do_y
		move.w	Yd(a5),d4
		bra.s	\no_y
\do_y:	; Update Y
	add.w	d2,d4
\no_y:

	; D2 = Correct direction
	cmp.w	#3,d0
	bhi.s	\do_x
		move.w	Xd(a5),d3
		bra.s	\no_x
\do_x:	; Update X
	swap	d2
	add.w	d2,d3
\no_x
	rts
;	bra	player::check_move

Move_player_table:
	dc.w	-3,-1	; X > Dest_X / Y > Dest_Y / Abs(X-Dest_x) > Abs(Y-Dest_y)
	dc.w	-1,-3	; X > Dest_X / Y > Dest_Y / Abs(X-Dest_x) < Abs(Y-Dest_y)
	dc.w	-3,01	; X > Dest_X / Y < Dest_Y / Abs(X-Dest_x) > Abs(Y-Dest_y)
	dc.w	-1,03	; X > Dest_X / Y < Dest_Y / Abs(X-Dest_x) < Abs(Y-Dest_y)

	dc.w	03,-1	; X < Dest_X / Y > Dest_Y / Abs(X-Dest_x) > Abs(Y-Dest_y)
	dc.w	01,-3	; X < Dest_X / Y > Dest_Y / Abs(X-Dest_x) < Abs(Y-Dest_y)
	dc.w	03,01	; X < Dest_X / Y < Dest_Y / Abs(X-Dest_x) > Abs(Y-Dest_y)
	dc.w	01,03	; X < Dest_X / Y < Dest_Y / Abs(X-Dest_x) < Abs(Y-Dest_y)


; Dereference le Nieme personnage a partir de son numero dans la pile des sprites
; Les personnages numero -1, -2 et -3 sont reservees pour acceder au personnages
; du joueur.
; In:
;	d0.w = Numero du personnage
;	d1.b = Nombre de repetition avant arret (1 => le 1er personnnage portant ce numero)
; Out:
;	a0.l -> Adresse dans la pile de sprites du personnage
;	d0.b = 2 (Trouve) / 1(trouve un, pas le Neme) / 0 (echec complet)
; Destroy:
;	a5/d0
player::deref_2_stack:
	movem.l	a1/d1-d3,-(a7)

	move.w	d0,d3
	moveq	#0,d0

	; Test personnages speciaux
	tst.w	d3
	bge.s	\standart
		; Recherches speciales
		ext.w	d1	; d1 ne peut pas etre > a 127 !
		
		; Recherche dans groupe courant ?
		addq.w	#1,d3	; d3 = -1 ?
		bne.s	\fail
			cmp.w	Member_group+Grp_nbr(a6),d1
			bhi.s	\fail
				mulu.w	#plf_struct,d1
				move.l	Member_adr(a6),a1
				adda.l	d1,a1
				move.w	plf_no(a1),d3	; plf_adr est peut-etre faux !
				moveq	#1,d1
				;bra.s	\standart
		; Recherche sources courantes ?
;		addq.w	#1,d3	; d3 = -2 ?
;		bne.s	\fail
		; Recherche destination courante
;		addq.w	#1,d3	; d3 = -3 ?
;		bne.s	\fail
		; Recherche autres ennemis
;		addq.w	#1,d3	; d3 = -4 ?
;		bne.s	\fail
		
\standart:

	move.l	Spr_print_adr(a6),a1

	; Boucle de recherche
\loop:		cmp.w	Player_no(a1),d3	; Un sprite a tres peu de chance
		bne.s	\next			; De passer ce test
			; On en a trouve un !
			move.l	a1,a0
			move.b	#1,d0		; Succe partiel
			subq.b	#1,d1
			beq.s	\success
\next:		adda.w	Size(a1),a1
		cmp.l	Spr_print_end_adr(a6),a1
		bcs.s	\loop			; Ou bmi ?
	; Echec de la recherche
	bra.s	\fail

\success:
	move.b	#2,d0	; Success !
\fail:
	movem.l	(a7)+,a1/d1-d3
	rts
