; Module 'TAGS POSITION'

; Gestion des tags de positions.
; Ces derniers permettent de passer d'un numero a une position a l'ecran.
; Definit dans la liste par :
;	dc.w	POSITION,NO_POSITION,EXTRA_1,EXTRA_2
; EXTRA_1 & EXTRA_2 sont des donnes inutilises pour le momment.


; Liste des fonctions :
;	position::deref
;	position::next
;	position::ref


; Reference une position par un numero de TAG de position
; Attention : elle recherche le plus proche TAG autour du coin !
; Retourne -1.l en cas d'echec
; In :
;	d0.w = X
;	d1.w = Y
; Out:
;	d2.w = Numero de position 
; Destroy :
;	
position::ref
	movem.l	d0-d1/d3-d7/a0-a3,-(a7)

	; Charge l'origine de la table d'item-hauteur
	move.l	Height_tab(a6),a0
	move.l	Items_list(a6),a3

	; Calc offset
	lsr.w	#4,d0
	lsr.w	#4,d1
	lsl.w	d7,d1
	add.w	d1,d0
	adda.w	d0,a0	; Origine de la case

	; Set direction table
	move.w	Map_x(a6),d0
	move.w	d0,d6
	move.w	d0,-(a7)	; Dy
	move.w	#1,-(a7)	; Dx
	neg.w	d0		
	move.w	d0,-(a7)	; -Dy
	move.w	#-1,-(a7)	; -Dx
	move.l	a7,a1		; A1 -> Direction table

	moveq	#2,d5		; Set R (=2*Rayon_carre+1 -1 pour dbf !)
	addq.w	#1,d6		; Pour remonter d'une case en haut a gauche

	move.w	#4,d7		; On fait au maximun 4 tours successifs (et pas de moveq !!)
\loop1:		; Calc Offset
		suba.w	d6,a0		; On retourne en haut a gauche
		moveq	#3,d1		; Table de direction a 4 elements
\loop15:		move.w	(a1)+,d0	; Lit la direction courante
			; Calc Tmp
			move.w	d5,d4	; On avance de 3, 5 ou 7 cases
\loop2:				clr.w	d2
				move.b	(a0),d2
				lsr.w	#3,d2		; /8 -> Numero Item
				subq.w	#1,d2		; Skip the 1 first item
				blt.s	\nxt
					lsl.w	#3,d2		; *8 -> Offset Item
					cmp.w	#I_POSITION,0(a3,d2.w)
					beq.s	\dir_found
\nxt				tst.w	d4
				beq.s	\last
					suba.w	d0,a0		; Next
\last:				dbf	d4,\loop2
			dbf	d1,\loop15
		subq.l	#8,a1		; Reset Direction Table
		addq.l	#2,d5		; Inc le rayon (3, 5 ou 7)
		dbf	d7,\loop1
	; Rien n'a ete trouve : on retourne -1
	moveq	#-1,d2
\return:
	addq.l	#8,a7
	movem.l	(a7)+,d0-d1/d3-d7/a0-a3
	rts
; On a trouve !
\dir_found:
	move.w	2(a3,d2.w),d2
	bra.s	\return


; Dereference un poisition en un numero d'item
; In :
;	d2.w = Numero de la position
; Out:
;	d1.w = Numero d'item
; Destroy:
;	d1/a0
deref_position_to_item:
	move.l	Items_list(a6),a0
	moveq	#1,d1			; D1 = Item
\loop_1:
		cmp.w	#I_POSITION,(a0)+
		bne.s	\next1
			cmp.w	(a0),d2
			beq.s	\found
\next1:		addq.l	#6,a0
		addq.w	#1,d1
		cmp.w	#32,d1
		bls.s	\loop_1
\error:	moveq	#-1,d1
\found:	rts



; Dereference une position a partir de son numero pour obtenir sa position
; reelle
; Si le numero n'est pas reference, on retourne les coordonnes (-1, -1)
; In :
;	d2.w = Numero de la position
; Out :
;	d0.w = X
;	d1.w = Y
; Destroy:
;	d0-d2/a0
position::deref
	; D'abord, on recherche le numero d'item de ce numero de position
	; dans la liste des items
	bsr.s	deref_position_to_item
	tst.w	d1
	blt.s	\error

	; On recherche cet item (d1) dans la table des Hauteurs/Items
	move.l	Height_tab(a6),a0
	move.w	Map_size(a6),d0
\loop_2:
		move.b	(a0)+,d2
		lsr.b	#3,d2
		cmp.b	d2,d1
		beq.s	\found2
		dbf	d0,\loop_2
	; Erreur le numero n'est pas reference
\error:
	moveq	#-1,d0
	moveq	#-1,d1
	bra.s	\fin
\found2
	; Cool : on a dans a0 l'adresse !
	move.l	a0,d0
	sub.l	Height_tab(a6),d0
	subq.w	#1,d0
	move.w	d0,d1
	; Calcul de y !
	lsr.w	d7,d1
	lsl.w	#4,d1
	; et celui de x !
	and.w	Mask_x(a6),d0
	lsl.w	#4,d0
\fin:	rts


; Recherche la position suivante.
; Ceci permet de faire une route.
; Exemple :
;	Si 1,2,3,4,5,7,8,9 sont definis, on a deux routes:
;	1,2,3,4,5 et 7,8,9.
; In :
;	d2.w = Numero de la position courante
; Out :
;	d0.w = X
;	d1.w = Y
;	d2.w = Numero de la position suivante
; Destroy:
;	d0-d2/a0

	; On doit revenir au debut de la boucle !
	; Recherche du "plus proche" minimun de la boucle
pos_next:	subq.w	#1,d2
		bsr	deref_position_to_item
		tst.w	d1
		bge.s	pos_next	; Il existe, on revient encore en arriere
position::next
	; Recherche position +1
	addq.w	#1,d2
	move.w	d2,-(a7)
	bsr	position::deref
	move.w	(a7)+,d2
	cmp.w	#-1,d0
	beq.s	pos_next
	rts

