; Module 'Print'
; Ensemble des routines d'affichages : plane + sprites.

; Liste des fonctions :
;	print::init
;	print::quit
;	print::mode
;	print::print

; Liste des modules :
;	exec_print

; Inialise le mode graphique.
; Entre autre alloue les deux plans et les tables Dhz / Hdz pour chacun de ces plans.
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	d0-d3/a0-a5
print::init:
	suba.l	a3,a3			; Adress of the plane-table
	suba.l	a4,a4			; Adress of the tile-table
	suba.l	a5,a5			; Adress of the indirect tile table
	moveq	#4,d3			; Size of one row (2^d3)

	; Init the planes.
	jsr	genlib::init_plane
	move.w	d0,Plane_A_hd
	beq	error::memory
	move.l	a0,Plan_A_adr(a6)	; Save the adress of the plane A

	jsr	genlib::init_plane
	move.w	d0,Plane_B_hd
	beq	error::memory
	move.l	a0,Plan_B_adr(a6)	; Save the adress of the plane B

	; Init the tab...
	move.w	#((256+128)*2),d0
	bsr	Alloc
	move.w	d0,Plane_tab_hd

print_reset_tab:
	movem.l	d0/a0-a1,-(a7)
	move.w	Plane_tab_hd(Pc),d0
	tios::DEREF	d0,a0
	move.l	a0,a1
	move.l	a0,Dhz_A_table(a6)
	lea	128(a0),a0
	move.l	a0,Dhz_B_table(a6)
	lea	128(a0),a0
	move.l	a0,Hdz_A_table(a6)
	lea	256(a0),a0
	move.l	a0,Hdz_B_table(a6)

	; Clear the tabs
	move.w	#(256+128)*2,d0
\clr_loop:
		clr.b	(a1)+
		dbf	d0,\clr_loop
	movem.l	(a7)+,d0/a0-a1
	rts


; Quitte le mode graphique.
; Libere les deux plans et les tables Dhz / Hdz pour cahcun de ces plans.
; In:
;	nothing
; Out:
;	nothing
; Destroy:
;	d0-d2/a0-a1
print::quit:
	move.w	Plane_A_hd(Pc),d0
	bsr	Free
	move.w	Plane_B_hd(Pc),d0
	bsr	Free
	move.w	Plane_tab_hd(Pc),d0
	bsr	Free
	clr.l	Plane_A_hd
	clr.w	Plane_tab_hd
	rts

Plane_A_hd:	dc.w	0	; Handle of plane A
Plane_B_hd:	dc.w	0	; Handle of plane B
Plane_tab_hd:	dc.w	0	; Handle of Dhz/Hdz tabs




; Initialise une facon d'afficher en fonction de la calculatrice, des effets,
; et initialise ce mode graphique (Description un peu vantarde :))
; In : d0.b = Mode (0,1,2  ...)
; B -> Plane B, A -> Plane A, Spr -> Sprite plane, LSpr -> Sprite plane (Listed sprites)
; SA -> Special Plane A, SB -> Special Plane B, TA -> Trans plane A, TB -> Trans plane B
;	%yy z xxxxx
; %xxxxx is
;	0 : B A Spr		; B
;	1 : A Spr B		; S
;	2 : A Spr .		; N
;	3 : Nothing + Spr	; C
;	4 : A Spr B Spr		; M
; %yy is
;	0: White transparent
;	1: Black transparent
;	2: White&Black special
;	3: ALl is transparent
; %z is
;	0: Normal update
;	1: Max update !
; Out : nothing
; Destroy: d0-d2
print::mode:
	bsr.s	print_reset_tab
	; Read mode
	move.b	d0,d2
	and.w	#%00111,d2
	lsl.w	#2,d2		; Mode 2 Mode on calc
	tst.b	Calc(a6)
	bne.s	\Ok
		addq.w	#2,d2		; Pour Ti-89
\Ok:	move.w	Print_table(Pc,d2.w),d2
	move.w	d2,Print_way(a6)

	; Read update
	move.l	#genlib::update_vscreen_16,-(a7)
	btst.l	#5,d0
	beq.s	\ok2
		move.l	#genlib::update_vscreen_max16,(a7)
\ok2	move.l	(a7)+,Update_plan(a6)

	; Read filter
	lsr.w	#6,d0
	jmp	genlib::set_filter


; Print the screen using the flag 'print_way'.
; In  : all data of 'print.h'
; Out : Update the current DScreen, the planes, ...
; Destroy : 
;	nothing
print::print:
	movem.l	d0-d7/a0-a6,-(a7)
	move.w	Print_way(a6),d2
	jsr	Print_table(Pc,d2.w)
	movem.l	(a7)+,d0-d7/a0-a6
	rts

; Module d'affichage
; In : a3 -> Linking buffer
exec_print:
	move.l	a3,-(a7)
	move.w	Print_way(a6),d2
	jsr	Print_table(Pc,d2.w)
	move.l	(a7)+,a3
	addq.l	#4,a3
	move.l	(a3)+,a0
	jmp	(a0)

Print_table:
	dc.w	Print1-Print_table
	dc.w	Print189-Print_table
	dc.w	Print2-Print_table
	dc.w	Print289-Print_table
	dc.w	Print3-Print_table
	dc.w	Print389-Print_table
	dc.w	PrintCls-Print_table
	dc.w	PrintCls-Print_table
	dc.w	Print4-Print_table
	dc.w	Print489-Print_table

	
; Print the screen on a Ti-92 & a Ti-89
; 1st : Clear screen
; 2nd : Sprites
PrintCls:
	clr.w	genlib::sprite_scr_x
	clr.w	genlib::sprite_scr_y

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	jsr	genlib::cls


	bra	print_sprite



; Print the screen on a Ti-92
; 1st : Plane B & no special effects
; 2nd : Plane A & no special effects
; 3rd : Sprites
Print1:
	move.l	Update_plan(a6),a1
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)
	move.l	Plan_B_adr(a6),a0	
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	move.l	Plan_B_adr(a6),a0	
	jsr	genlib::put_plane
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_fgrd_plane

	bra	print_sprite

; Print the screen on a Ti-92
; 1st : Plane A & no special effects
; 2nd : Sprites
; 2rd : Plane B & no special effects
Print2:
	move.l	Update_plan(a6),a1
	move.l	Plan_B_adr(a6),a0
	jsr	(a1)
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_plane

	bsr	print_sprite
	
	move.l	Plan_B_adr(a6),a0
	jmp	genlib::put_fgrd_plane


; Print the screen on a Ti-92
; 1st : Plane A
; 2nd : Sprites
Print3:
	move.l	Update_plan(a6),a1
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_plane
	bra	print_sprite



; Print the screen on a Ti-92
; 1st : Plane A & no special effects
; 2nd : Sprites <0
; 3rd : Plane B & no special effects
; 4em : Sprites >0
Print4:
	move.l	Update_plan(a6),a1
	move.l	Plan_B_adr(a6),a0
	jsr	(a1)
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_plane

	bsr	print_sprite_before
	
	move.l	Plan_B_adr(a6),a0
	jsr	genlib::put_fgrd_plane

	bra	print_sprite_after
	
;///////////////////////////////////////////////////////////////////////

; Print the screen on a Ti-89
; 1st : Plane B & no special effects
; 2nd : Plane A & no special effects
; 3rd : Sprites
Print189:
	move.l	Update_plan(a6),a1
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)
	move.l	Plan_B_adr(a6),a0	
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	; Plane B
	move.l	Plan_B_adr(a6),a0	
	jsr	genlib::put_plane_89

	; Plane A
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_fgrd_plane_89

	bra	print_sprite


; Print the screen on a Ti-89
; 1st : Plane A & no special effects
; 2nd : Sprites
; 2rd : Plane B & no special effects
Print289:
	move.l	Update_plan(a6),a1
	move.l	Plan_B_adr(a6),a0
	jsr	(a1)
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	; Plane A
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_plane_89

	bsr	print_sprite
	
	; Plane B
	move.l	Plan_B_adr(a6),a0
	jmp	genlib::put_fgrd_plane_89


; Print the screen on a Ti-89
; 1st : Plane A
; 2nd : Sprites
Print389:
	move.l	Update_plan(a6),a1
	; Plane A
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)

	bsr.s	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_plane_89

	; Sprites
	bra	print_sprite


; Print the screen on a Ti-89
; 1st : Plane A & no special effects
; 2nd : Sprites <0
; 3rd : Plane B & no special effects
; 4em : Sprites >0
Print489:
	move.l	Update_plan(a6),a1
	move.l	Plan_B_adr(a6),a0
	jsr	(a1)
	move.l	Plan_A_adr(a6),a0
	jsr	genlib::copy_xyplane_xysprite	; Update scrolling data of sprite-plane
	jsr	(a1)

	bsr	Trie_sprite
\wait:	tst.l	genlib::timer
	beq.s	\wait

	move.l	Plan_A_adr(a6),a0
	jsr	genlib::put_plane_89

	bsr	print_sprite_before
	
	move.l	Plan_B_adr(a6),a0
	jsr	genlib::put_fgrd_plane_89

	bra	print_sprite_after


;///////////////////////////////////////////////////////////////////////

; Classe les sprites dans CAni_table
Trie_sprite:
	move.l	Spr_print_adr(a6),a5		; a5 -> Sprite buffer
	move.l	Spr_print_end_adr(a6),d5	; D5 = Adress of the end of the sprite buffer

	moveq	#-MAX_SPRITE_SIZE,d4	; D4 pour clipping X
	moveq	#-MAX_SPRITE_SIZE,d6	; D6 pour clipping Y
	move.l	Plan_A_adr(a6),a0
	add.w	PLANE_XS(a0),d4
	add.w	PLANE_YS(a0),d6

	moveq	#MAX_CANI,d3		; MAX_CANI - Nombre d'item entree dans la table
	lea	CAni_table(a6),a4	; A4 -> Table des sprites tries
	move.l	a4,a3			; Pour usage ulterieur

	cmp.l	d5,a5			; End of the sprite buffer ?
	bcc.s	\end			; Yes so exit (No sprite)

\next		tst.b	Drawing(a5)		; Get the print-type of the sprite
		beq.s	\no			; >=0 or <=0 ?
			move.w	X(a5),d0		; No, so read X
			sub.w	d4,d0
			cmp.w	#SCREEN_X*8+2*MAX_SPRITE_SIZE,d0
			bhi.s	\no			; Clipping X (Grossier !)
				move.w	Y(a5),d1		; and Y
				sub.w	d6,d1
				cmp.w	#SCREEN_Y+2*MAX_SPRITE_SIZE,d1
				bhi.s	\no			; Clipping Y (Grossier !)
					subq.w	#1,d3
					beq.s	\end		; Sprite overflow !
;					add.w	Hz(a5),d1	; Il faut gerer la hauteur
					move.w	d1,(a4)+	; Copie Y
					move.l	a5,(a4)+	; And the adress of the sprite
\no:		adda.w	Size(a5),a5		; Next sprite 
		cmp.l	d5,a5			; End of the sprite buffer ?
		bcs.s	\next			; No so continue the loop
\end:
	; Fin de la recopie

	; Calcul du nbr d'element dans le tableau.
	neg.w	d3
	add.w	#MAX_CANI,d3

; IN : A3 -> CAni_tbale et d3.w = Nbr d'element
Tri:
	subq.w	#1,d3
	move.w	d3,CAni_nbr(a6)
	blt.s	Nothing

	move.w	d3,d0
	; Trie bulle (Pas terrible certes mais on en a pas beaucoup a classer !)
\loop_i:
		move.w	d0,d1
		move.l	a3,a4
		bra.s	\start
		; Recherche minimun (d2.w, -> a2)
\loop_j:		cmp.w	(a4),d2
			ble.s	\Ok
\start:				move.w	(a4),d2
				move.l	a4,a2
\Ok:			addq.l	#cani_struct_size,a4
			dbf.s	d1,\loop_j
		; Sauvegarde minimun
		move.w	(a3),d4
		move.w	d2,(a3)+
		move.w	d4,(a2)+
		move.l	(a3),d4
		move.l	(a2),(a3)+
		move.l	d4,(a2)+

		dbf.s	d0,\loop_i
	rts

;/*****************************************************\
;|							|
;|		 	Print sprites			|
;|							|
;\*****************************************************/
print_sprite:
	lea	(CAni_table+cani_adr)(a6),a4
	move.w	CAni_nbr(a6),d3
	blt.s	Nothing
\Ploop:
		move.l	(a4),a5
		addq.l	#cani_struct_size,a4
	
		move.b	Drawing(a5),d2		; Get the print-type of the sprite
		andi.w	#$007F,d2
		move.w	X(a5),d0		; Read X
		move.w	Y(a5),d1		; and Y
		move.w	Print_sprite_table(Pc,d2.w),d2		; Get the offset
		jsr	Print_sprite_table(Pc,d2.w)		; jump to the sub-routine
\next:		dbf	d3,\Ploop
Nothing	rts

print_sprite_before:
	lea	(CAni_table+cani_adr)(a6),a4
	move.w	CAni_nbr(a6),d3
	blt.s	Nothing
\Ploop:
		move.l	(a4),a5
		addq.l	#cani_struct_size,a4
	
		move.b	Drawing(a5),d2		; Get the print-type of the sprite
		bge.s	\next
			andi.w	#$007F,d2
			move.w	X(a5),d0		; Read X
			move.w	Y(a5),d1		; and Y
			move.w	Print_sprite_table(Pc,d2.w),d2		; Get the offset
			jsr	Print_sprite_table(Pc,d2.w)		; jump to the sub-routine
\next:		dbf	d3,\Ploop
	rts

	; Print sprites
print_sprite_after:
	lea	(CAni_table+cani_adr)(a6),a4
	move.w	CAni_nbr(a6),d3
	blt.s	Nothing
\Ploop:
		move.l	(a4),a5
		addq.l	#cani_struct_size,a4
	
		move.b	Drawing(a5),d2		; Get the print-type of the sprite
		ble.s	\next
			andi.w	#$007F,d2
			move.w	X(a5),d0		; Read X
			move.w	Y(a5),d1		; and Y
			move.w	Print_sprite_table(Pc,d2.w),d2		; Get the offset
			jsr	Print_sprite_table(Pc,d2.w)		; jump to the sub-routine
\next:		dbf	d3,\Ploop
	rts

Print_sprite_table:
	dc.w	Nothing-Print_sprite_table
	dc.w	Ani_Bgs-Print_sprite_table
	dc.w	Bgs-Print_sprite_table
	dc.w	Ani_S16-Print_sprite_table
	dc.w	S16-Print_sprite_table

; Print a BGs sprite with animation
; In: A5 -> Animation_struct
Ani_Bgs:
	; Calcul du numero de l'image grace a l'animation
	move.l	Ani_adr(a5),a0	; A0 -> Animation_tab (tableau d'animation_item)
	clr.w	d2
	move.b	Ani(a5),d2
	adda.w	d2,a0		; A0 -> Animation_item

	move.b	Ani_sub(a5),d2	
	subq.b	#1,Ani_cpt(a5)
	bhi.s	\ani_stop
		addq.b	#4,d2		; Next sub-animation (Up / Down / left / right) : atteindre le suivant
		cmp.b	2(a0),d2
		bcs.s	\ouf
			clr.w	d2
\ouf:		move.b	1(a0),Ani_cpt(a5)
		move.b	d2,Ani_sub(a5)
\ani_stop:
	add.b	(a0),d2			; D2 += Ani_base_image

; Ici :
;	 d2.w = Numero du sprite en Bgs a afficher (Sans correction du sens)
Bgs_print:
	add.b	Sens(a5),d2		; D2 += Sens (0, 1, 2 or 3) correspondant a Haut, Bas, Gauche et droite

	move.l	Gfx_adr(a5),a1		; A1 -> *Gfx_table
	lsl.w	#2,d2			; D2 *= 4
	adda.w	d2,a1			; A1 -> Gfx_Bgs_struct
	moveq	#0,d2			; D2.w = 0
	move.b	gfx_flag(a1),d2		; D2 = Fliph_h(0 ou 2) | Flip_v (0 ou 4)
	move.l	Bgs_table(Pc,d2.w),a2	; A2 -> Routine a executer pour afficher le sprite
	lea	Free_buffer(a6),a0	; A0 -> Free buffer pour flipping
	move.b	gfx_deca_x(a1),d2	; D2.b = Deca_x
	ext.w	d2			; D2.w = Deca_x
	sub.w	d2,d0			; Correct the x value
	adda.w	gfx_offset(a1),a1	; A1 -> Bgs graph
	moveq	#0,d2			; D2.w = 0
	move.b	(a1),d2			; D2.w = Size y
	sub.w	d2,d1			; Correct d1
	addq.w	#6,d1			; Y+6 pour corriger un pb de perspective.
	jmp	(a2)

Bgs_table:
	dc.l	Bgs_solve
	dc.l	genlib::put_big_sprite_flip_h
	dc.l	genlib::put_big_sprite_flip_v
	dc.l	genlib::put_big_sprite_flip_hv

Bgs_solve:
	move.l	a1,a0			; Petit probleme d'argument ave put_big_sprite
	jmp	genlib::put_big_sprite

; Print a BGs sprite with no animation
; In: A5 -> Animation struct
Bgs:
	move.b	Ani(a5),d2
	bra.s	Bgs_print



; Print a S16 sprite with animation
; In: A5 -> Animation struct
Ani_S16:

; Print a S16 sprite with no animation
; In: A5 -> Animation struct
S16:	move.b	Ani(a5),d2
	jmp	genlib::put_sprite_16






