; Module 'routines'
; Des routines utiles pour tout le monde...

; Liste des fonctions:
;	put_number
;	random
;	GetKey
;	sprintf
;	Clear_LCD_MEM
;	Alloc
;	Free
;	push_hd
;	Display_string
;	InputStr
;	idle_loop

; Alloue de la place.
; le nombre d'octets a allouer est dans d0.w (Oui un word !)
; S'il n'y a pas assez de place, il quitte sur une erreur de memoire
; Utilise la fonction standart HeapAlloc !
; Entree:
;	D0.w = Octets a allouer
; Retourne:
;	d0.w =  Handle
;	A0.l -> Buffer
; Detruit :
;	d0-d2/a0-a1
Alloc:
	move.w	d0,-(a7)
	clr.w	-(a7)
	jsr	tios::HeapAlloc
	addq.l	#4,a7
	tst.w	d0
	beq	error::memory
	move.w	d0,d1
	tios::DEREF	d1,a0
	rts

; Libere l'handle d0.w
; (Surtout pour economiser un appel long...)
; Si l'handle est nul, ne fait aucun appel a tios::HeapFree
; In :
;	d0.w = Handle
; Out :
;	nothing
; destroy :
;	nothing
Free:
	movem.l	d0-d2/a0-a1,-(a7)
	move.w	d0,-(a7)
	beq.s	\no
	jsr	tios::HeapFree
\no	addq.l	#2,a7
	movem.l	(a7)+,d0-d2/a0-a1
	rts

; Sauvegarde des handle
; In:
;	d0.w = Handle
; Out
;	nothing
; Destroy:
;	nothing
push_hd:
	jmp	genlib::push_hd


; Put a string on the screen
; In:
;	a0 -> String
;	d0.w = x
;	d1.w = y
; Out :
;	nothing
; Destroy:
;	nothing
Display_string:
	movem.l	d0-d2/a0-a1,-(a7)
	move.w	#4,-(a7)
	move.l	a0,-(a7)
	move.w	d1,-(a7)
	move.w	d0,-(a7)
	jsr	tios::DrawStrXY
	lea     10(a7),a7
	movem.l	(a7)+,d0-d2/a0-a1
	rts

; Get a key from the key buffer
; In :
;	nothing
; Out :
;	d0.w = Key code
; Destroy :
;	d0
idle_loop:
	movem.l	d1-d2/a0-a1,-(a7)
	clr.w	-(a7)
	clr.l	-(a7)
	jsr	tios::GKeyIn
	addq.l	#6,a7
	movem.l	(a7)+,d1-d2/a0-a1
	rts


; Input a string
; In :
;	d1.w = x
;	d2.w = y
;	d3.w = maxchar
; Out:
;	d0.w = string lenght
;	a0.l = adress of the string
; Destroy :
;	d0/a0
InputStr:
	movem.l	d1-d4/a1,-(a7)

	movem.l	d1-d2,-(a7)	; Save x-y

	move.b	#1,tios::MaxHandles+$24+$11	;ALPHA LOCK

	lea	Free_buffer+100(a6),a0		; a0 -> String

	move.l	a0,a1				; a1 -> Current char
	clr.w	d4				; 0 char
	
\loop:	
	move.b	#' ',(a1)			; Space
	clr.b	1(a1)				; Null string
	movem.l	(a7),d0-d1			; Read x-y
	bsr.s	Display_string
	bsr.s	idle_loop
	cmp.w	#13,d0		;Enter ?
	beq.s	\Enter
	cmp.w	#257,d0		;Clear ?
	beq.s	\Del
	cmp.w	#264,d0		;ESC ?
	beq.s	\Esc
	cmp.w	#255,d0		;Charactre invalide ?
	bhi.s	\loop
	cmp.w	d3,d4		;Maxchar ?
	beq.s	\loop
	move.b	d0,(a1)+
	addq.w	#1,d4
	bra.s	\loop

\Del:
	tst.w	d4
	beq.s	\loop
	subq.w	#1,d4
	clr.b	-(a1)
	bra.s	\loop

\Esc:
	clr.w	d4		; No char
	clr.b	(a0)		; Null string

\Enter:
	clr.b	(a1)		; Null string
	move.w	d4,d0
	addq.l	#8,a7		; Skip x.l / y.l
	movem.l (a7)+,d1-d4/a1
	rts

; Convertit une chaine en un nombre
; In:
;	a0 -> chaine (NULL)
; Out:
;	d0.l = Nombre
; Destroy:
;	d0-d1/a0
strtonbr:
	moveq	#0,d0
\loop		moveq	#0,d1
		move.b	(a0)+,d1
		beq.s	\end
		sub.b	#'0',d1
		mulu.w	#10,d0
		add.l	d1,d0
		bra.s	\loop
\end:	rts

; Efface l'ecran LCD_MEM
; In :
;	nothing
; Out :
;	nothing
; Destroy :
;	nothing
Clear_LCD_MEM:
	movem.l	d0/a0,-(a7)
	lea	LCD_MEM,a0
	move.w	#960-1,d0
\loop:		clr.l	(a0)+
		dbf	d0,\loop
	movem.l	(a7)+,d0/a0
	rts

; Fonction standart du tios
; Un appel a ceci permet d'economiser 4 octets par appel (+8)
sprintf	jmp	tios::sprintf


; Lit une touche du clavier
; In:
;	nothing
; Out:
;	d0.w = Code de la touche
; Destroy :
;	d0
; Code des touches :
;	00 : ESC
;	01 : APPS
;	02 : PLUS
;	03 : MINUS
;	04 : LEFT
;	05 : UP
;	06 : RIGHT
;	07 : DOWN
;	08 : F1 / 2nd
;	09 : F2 / Diamond
;	10 : F3 / Home
;	11 : F4 / X
;	12 : F5 / Shift
;	13 : F6 / Alpha
;	14 : F7 / Mode
;	15 : F8 / Y
GetKey:
	movem.l	d1/a0,-(a7)
	moveq	#-1,d1
	move.l	d1,key_mask(a6)
\Cont:	jsr 	genlib::wait_a_key
	jsr	genlib::read_joypad
	moveq	#15,d1
\loop:		btst.l	d1,d0
		beq.s	\end
		dbf	d1,\loop
	bra.s	\Cont

\end:	move.w	d1,d0
	movem.l	(a7)+,d1/a0
	rts

; Put the number d2.l at coordinates ( d3.w = X, d4.w = Y) with (d1).w digits
; on genlib::window
; In:
;	d3.W = X
;	d4.w = Y
;	d1.w = Number of digits
; Out :
;	nothing
; Destroy :
;	nothing
put_number:
Putnumber:
PutNumber:
Put_number:
	movem.l d0-d2/a0-a1,-(a7)

	lea	(Free_buffer+100)(a6),a0	; Load tempory storage string adress
	clr.b	(a0)		; for DrawStrXY
\Rep1:
		divu	#10,d2		; d3 / 10
		swap	d2		; Work on the rest
		addi.b	#'0',d2		; Add 48 to the rest of the division
		move.b	d2,-(a0)	; Save the rest
		subq.w	#1,d1		
		beq.s	\CSDone		; to avoid a crash if the number has too many digits
		clr.w	d2		; Clear the rest
		swap	d2		; New number D2.l = Int(D2.l / 10)
		bne.s	\Rep1		; No need of 'tst' because 'swap' does it

	tst.w  d1
	beq.s	\CSDone	
		subq.w	#1,d1			; For dbf
\Rep2:			move.b #'0',-(a0)	; Complete par des 0 (si on veut des espaces : ' ')
			dbra.s	d1,\Rep2
\CSDone:
	; Here X=d3, Y=d4 and A0 -> String
	move.w	d3,d0
	move.l	d4,d1
	move.l	genlib::window,a1		; Or ?
	jsr	genlib::put_large_string	; Or ?
\Stat1:
	movem.l (a7)+,d0-d2/a0-a1
	rts


; Random number between (0...D0-1).
; In :
;	D0.w = Max
; Out :
;	d0.l = Random value
; Destroy:
;	Nothing
; Note: original routine made by David E.
random:
	move.l  d1,-(a7)		; saves d1
	move.w  rand_seed(a6),d1	; d1 = rand_seed
	mulu.w  #31421,d1		; rand_seed * 31421
	add.w   #6927,d1		; + 6927
	mulu.w  d1,d0			; * rand _ max
	move.w  d1,rand_seed(a6)	; new rand_seed = d1
	clr.w   d0			; d0.w = 0
	swap    d0			; swap
	move.l  (a7)+,d1		; restores d1
	rts
