; Search
; ensemble des routines necessaires pour rechercher un fichier.

;Liste des fonctions :
;	search::file
;	search::file2
;	search::graph_file
;	search::rom

; Recherche la chaine de caracteres pointee par a0.
; In  :
;	A0 -> File_str
; Out :
;	D0.w = Handle of the folder or H_NULL
;	D1.w = Handle of the file or ????
;	A1.l -> File or ???
; If there is an error, d0.w = 0 !
; Destroy : d0/d1/a1
search::file
	movem.l	d2/a0,-(a7)
	moveq	#1,d1
	jsr	filelib::search
	move.w	SYM_ENTRY.hVal(a0),d1
	move.w	d1,d2
	tios::DEREF d2,a1
	movem.l	(a7)+,d2/a0
	rts

; Search and extract graph-file (->a0)
; In : a0 -> File_str
; Out : a0 -> Graph_data
; Destroy : d0-d2/a0-a1
; If it can't find it, it will quit the program.
; Save the handle on internal genlib handle stack.
search::graph_file:
	move.l	#GFX_SIGNATURE,d2
	bsr.s	search::file2
	addq.l	#2+4,a0			; Skip the size of the string and the signature
	suba.l	a1,a1			; A1 = 0

; Extract the first archive to a1 or create a new handle
; Input : a0 -> Archive data compressed with shrnklib
;	  a1 -> Where to extract (Shrnklib format !)
; Output : a0 -> Archive and d2.w the new handle if a1=0
; Destroy d0-d2 / a0
do_shk:
	jsr	shrnklib::OpenArchive
	tst.w	d0
	beq	error::memory		; memory
	moveq	#0,d1	; 1st archive
	move.l	a1,a0	
	jsr	shrnklib::Extract
	cmp.l	a0,d1	; D1 still null !
	beq	error::memory		; memory
	jsr	shrnklib::CloseArchive
	cmp.l	a1,d1	; A1 = 0 so there is a new handle !
	bne.s	\no_do
		move.w	d2,d0
		jsr	genlib::push_hd
		tios::DEREF	d0,a0	; because TiOs may change memory, even it seems it never appears
\no_do:	rts

; Search and check for signature
; In  : A0 -> File_str
;	D2.l = Signature
; Out : A0 -> File
; If there is an error, it will end the program (Invalid signature or file not found)
; Destroy : d0/d1
search::file2:
	movem.l	d0-d1/a1,-(a7)
	move.l	a0,a1
	moveq	#1,d1
	jsr	filelib::search
	move.w	SYM_ENTRY.hVal(a0),d1
	move.l	a1,a0				; For error routines
	tst.w	d0
	beq	error::file_not_found		; File not found
	tios::DEREF d1,a1
	cmp.l	2(a1),d2
	bne	error::file_invalid		; Invalid file
	move.l	a1,a0
	movem.l	(a7)+,d0-d1/a1
	rts

; Recherche l'ensemble des fichiers externes
; Le fichier des personnages,
; le fichier des objets et
; le fichier des evenements
; et met a jour les variables pointant sur les tableaux ROM des Objets, des histoires et des personnages
; i.e. ROM_Object_tab, ROM_Story_tab & ROM_Player_tab
; In / Out: nothing
; Destroy : d0-d2/a0-a1
; Note :
;	If it can't found some files, it will quit the program
search::rom:
	move.l	#ROM_SIGNATURE,d2
	lea	Rom_str(Pc),a0
	bsr.s	search::file2
	addq.l	#2+4,a0			; Skip the size of the string and the signature
	move.l	a0,ROM_Object_tab(a6)
	lea	object_rom_size*MAX_OBJECT(a0),a0
	move.l	a0,ROM_Player_tab(a6)

	move.l	#STORY_SIGNATURE,d2
	lea	Rom_story_str(Pc),a0
	bsr.s	search::file2
	addq.l	#2+4,a0			; Skip the size of the string and the signature
	move.l	a0,ROM_Story_tab(a6)

	rts

; Chaine de caractere contenant le fichier ROM
Rom_str:
	dc.b "ctdata",0
Rom_player_str:
	dc.b "ctplayer",0
Rom_story_str:
	dc.b "ctstory",0
Rom_object_str:
	dc.b "ctobject",0
	EVEN
